import sys, os, argparse, gzip

from SBI.beans           import Path
from SBI.databases import Uniprotlink

from archdb.src import Source, Uniprot
from archdb.src import start_transaction, end_transaction

# User Options
def options(*args, **kwds):
    parser = argparse.ArgumentParser(prog = 'Uniprot2SQL',  formatter_class = argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument('-d', '--database',     dest     = "database",
                        action = "store",       required = True,  help = "Destination Directory for Uniprot database.")
    parser.add_argument('-s', '--sql',          dest     = "sqlfile",
                        action = "store",       required = True,  help = "Output SQL file pattern (_)")
    parser.add_argument('-k', '--skipdownload', dest     = "skip_download",
                        action = "store_true",  default  = False,  help = "Skips downloading the database.")
    parser.add_argument('-v', '--verbose',      dest     = "verbose",
                        action = "store_true",  default  = False, help = "Verbose Mode.")
    options = parser.parse_args()
    return options

def Uniprot2SQL(database, sqlfile, skip_download, verbose):
    uniprot_connect = Uniprotlink(local = database)
    newsource       = None

    if not options.skip_download:
        if verbose: sys.stderr.write("Downloading Uniprot database to {0} ...\n".format(database))
        uniprot_connect.download()
        newsource = Source(name = 'uniprot', source = uniprot_connect.source)
        if verbose: sys.stderr.write("Download Finished.\n")
    else:
        if verbose: sys.stderr.write("Using previously downloaded database.\n")

    file_counter  = 1
    file_sequence = 0
    file_sql_name = sqlfile.replace('_','{0:03}')

    if verbose: sys.stderr.write("Parsing Uniprot.\n")
    if verbose: sys.stderr.write("Writing {0} ....\n".format(file_sql_name.format(file_counter)))
    Path.mkdir(os.path.split(os.path.abspath(sqlfile))[0])
    sql_fd = gzip.open(file_sql_name.format(file_counter), 'wb')
    sql_fd.write(start_transaction())
    if newsource is not None:
        sql_fd.write(newsource.toSQL())

    for uni_line in uniprot_connect.localUniprots:
        newuni = Uniprot(inline = uni_line)
        if file_sequence > 500000:
            sql_fd.write(end_transaction())
            sql_fd.close()
            file_sequence = 0
            file_counter += 1
            if verbose: sys.stderr.write("Writing {0} ....\n".format(file_sql_name.format(file_counter)))
            sql_fd = gzip.open(file_sql_name.format(file_counter), 'wb')
            sql_fd.write(start_transaction())

        sql_fd.write(newuni.toSQL())
        file_sequence += 1
    if verbose: sys.stderr.write("End execution.\n")

if __name__ == "__main__":
    options = options()
    Uniprot2SQL(options.database, options.sqlfile, options.skip_download, options.verbose)



'''
@file: HmmResult.py

@author: Jaume Bonet
@mail:   jaume.bonet@gmail.com
@date:   2014

@ [oliva's lab](http://sbi.imim.es)

@class: HmmResult
'''
import re

from SBI.beans import StorableObject


class HmmResult(StorableObject):
    '''
    Contains the results of a HMMER output file.

    '''
    _PFAM = re.compile('pfam', re.IGNORECASE)
    _CATH = re.compile('cath', re.IGNORECASE)

    def __init__(self, query_name, query_sequence, database = None):
        '''
        @param:    query_name
        @pdef:     name of the query protein
        @ptype:    {String}

        @param:    query_sequence
        @pdef:     sequence of the query protein
        @ptype:    {String}

        @param:    database
        @pdef:     name of the queried database
        @ptype:    {String}
        '''
        self._qname = query_name
        self._qseq  = query_sequence
        self._db    = database
        self._hits  = []

    ##############
    # ATTRIBUTES #
    ##############
    @property
    def query_name(self):
        '''
        @return: {String}
        '''
        return self._qname

    @property
    def query_sequence(self):
        '''
        @return: {String}
        '''
        return self._qseq

    @property
    def database(self):
        '''
        @return: {String}
        '''
        return self._db

    @property
    def hits(self):
        '''
        @return: {List} of {HmmHit}
        '''
        return self._hits

    ############
    # BOOLEANS #
    ############
    @property
    def is_pfam(self):
        '''
        Check if we queried against a PFAM database

        @return: {Boolean}
        '''
        return bool(self._PFAM.search(self._db))

    @property
    def is_cath(self):
        '''
        Check if we queried against a CATH database

        @return: {Boolean}
        '''
        return bool(self._CATH.search(self._db))

    @property
    def has_domains(self):
        '''
        Check if any hit was found.

        @return:{Boolean}
        '''
        return len(self.hits) > 0

    @property
    def is_empty(self):
        '''
        Check if no hit was found.

        @return:{Boolean}
        '''
        return not self.has_domains

    ###########
    # METHODS #
    ###########
    def add_hit(self, hit):
        '''
        append a new hmm hit result to the object.

        @param:    hit
        @pdef:     new hit to add
        @ptype:    {HmmHit}
        '''
        self._hits.append(hit)

    def get_hits(self, evalue = None, coverage = None, overlap = None):
        '''
        Retrieve hits that fulfill different filters.

        @param:    evalue
        @pdef:     maximum evalue allowed
        @ptype:    {Float}

        @param:    coverage
        @pdef:     minimum coverage of the assigned hit required
        @ptype:    {Float}

        @param:    overlap
        @pdef:     maximum overlap to previous retrieved hits allowed
        @ptype:    {Float}

        @return: {List} of {HmmHit}
        '''
        if evalue is None and coverage is None and overlap is None:
            return self.hits
        else:
            hits = []
            coverage = float(0) if coverage is None else float(coverage)
            evalue   = float(1000) if evalue is None else float(evalue)
            for domain in self.hits:
                if evalue >= domain.evalue and coverage <= domain.coverage:
                    if overlap is None:
                        hits.append(domain)
                    else:
                        addDomain = True
                        for previous_domain in hits:
                            if previous_domain.overlap(domain) > overlap:
                                addDomain = False
                        if addDomain:
                            hits.append(domain)
            return hits

    def get_used_hits(self):
        '''
        Retrieve those hits marked as used.

        @return: {List} of {HmmHit}
        '''
        hits = []
        for domain in self.hits:
            if domain.used:
                hits.append(domain)
        return hits



import sys
import argparse
import os
import re
from Bio import SeqIO as SeqIE
from Bio import ExPASy
from Bio.Seq import Seq
from Bio import SeqRecord
from Bio.Alphabet import IUPAC
import ConfigParser
import itertools
import shutil
import subprocess
import time
import random


# Add '.' to sys.path
src_path = os.path.abspath(os.path.dirname(__file__))
sys.path.append(src_path)
script_path = src_path+"/../scripts"
sys.path.append(script_path)

# Read configuration file
config = ConfigParser.ConfigParser()
config_file = os.path.join(script_path, 'config.ini')
config.read(config_file)

# Add SBI library path to sys.path
src_path = os.path.join(config.get('Paths','modppi_path'),config.get('Paths','sbi_library_path'))
sys.path.append(src_path)

# Add functions path to sys.path    
src_path = os.path.join(config.get('Paths','modppi_path'),config.get('Paths','functions_path'))
sys.path.append(src_path)

# Import functions
import functions
from SeqIO import *
from functions import *
from SBI.external.blast import blast_parser
from SBI.structure.contacts import Complex
from SBI.sequence import Sequence
from SBI.structure import PDB

def fileExist (file):
 return os.path.exists(file) and os.path.isfile(file)


if len(sys.argv)==1:
 sys.stdout.write("Excution is 'select_cluster <directory> <score>' where directory is the output folder of modelist.py and score is the score grouping of Rosetta to test (ddG_all by default)\n")
 exit(0)

input_dir=sys.argv[1]

if len(sys.argv)>2: score=sys.argv[2]
else: score="ddG_all"


files=os.listdir(input_dir)

ppi_sets=set()
for input_file in files:
   if input_file.endswith(".ppi"): 
      ppi_sets.add(input_file)

comparison={}
for file_ppi in ppi_sets:
    ppis=[]
    fd=open(os.path.join(input_dir,file_ppi),'r')
    for line in fd:
      pair=line.split()
      ppis.append((pair[0],pair[1]))
    fd.close()
    root_ppi=file_ppi.rstrip(".ppi")
    data=root_ppi.split("_")
    label=data[0]
    wt=(data[1],data[-1])
    ddg_file=root_ppi+"."+score+"_distribution_statistic.out"
    print "Check file  %s"%os.path.join(input_dir,ddg_file)
    if fileExist(os.path.join(input_dir,ddg_file)):
      print "Open %s"%os.path.join(input_dir,ddg_file)
      fd=open(os.path.join(input_dir,ddg_file),'r')
      for line in fd:
       pair=line.strip().split("versus")
       statistic=line.strip().split()
       if len(statistic)>1:
         if statistic[-1] == "Statistic": statistic_type=statistic[0]
       if len(pair)>1:
        ppia=pair[0].split()[0]
        ppib=pair[1].split()[0]
        p,z=ppia.split("-")
        q,conformer_a= z.split("#")
        cluster_a,interface_a= conformer_a.split("_")
        overlap_a=int(interface_a.lstrip("i"))
        x,z=ppib.split("-")
        y,conformer_b = z.split("#")
        cluster_b,interface_b= conformer_b.split("_")
        overlap_b=int(interface_b.lstrip("i"))
        if int(cluster_a) == int(cluster_b): 
          if (p,q) == wt or (q,p) == wt :
           if overlap_b > 0:
            if x>y:  interactors= x+"::"+y
            else:    interactors= y+"::"+x 
            comparison.setdefault((interactors,statistic_type),[]).append((overlap_b,cluster_b,line.strip()))
          if (x,y) == wt or (y,x) == wt  :
           if overlap_a >0:
            if p>q:  interactors= p+"::"+q
            else:    interactors= q+"::"+p 
            comparison.setdefault((interactors,statistic_type),[]).append((overlap_a,cluster_a, line.strip()))
      fd.close()

for (file_name,analysis),result in comparison.iteritems(): 
  fo=open(os.path.join(input_dir,"Select_Cluster_"+score+"_"+file_name+"_"+analysis+".out"),"w") 
  print "Write results in %s"%(os.path.join(input_dir,"Select_Cluster_"+score+"_"+file_name+"_"+analysis+".out"))     
  for overlap,cluster,data in sorted(result,key=lambda x: x[0],reverse=True):
    #print "Percentage mutation %s Cluster %s  Data %s \n"%(overlap,cluster,data)
    fo.write("Percentage of mutation in interface around %d %s   Cluster %s  Data %s \n"%(10*int(overlap),"%",cluster,data))
  fo.close()
         

    




import os, sys, random
import numpy as np

class IC_scores(object):
    '''
    Compute the IC (inter-residue contacs) as described in Bonvin eLife 2015
    '''
    def __init__(self, interaction):
        '''
        Contructor
        '''
        self.interaction = interaction
        interacting_residues = self.interaction.get_interacting_residues(c_type = 'MIN', max_distance = 5.5, uniq=False)
        self.contact_pos1 = interacting_residues[0]
        self.contact_pos2 = interacting_residues[1]
        self.contact_distances = interacting_residues[2]
        self.res_types = {'HIS' : 'charged',
                          'ARG' : 'charged',
                          'LYS' : 'charged',
                          'ASP' : 'charged',
                          'GLU' : 'charged',
                          'SER' : 'polar',
                          'THR' : 'polar',
                          'ASN' : 'polar',
                          'GLN' : 'polar',
                          'CYS' : 'apolar',
                          'GLY' : 'apolar',
                          'PRO' : 'apolar',
                          'ALA' : 'apolar',
                          'VAL' : 'apolar',
                          'ILE' : 'apolar',
                          'MET' : 'apolar',
                          'LEU' : 'apolar',
                          'PHE' : 'apolar',
                          'TYR' : 'apolar',
                          'TRP' : 'apolar'  }

    def calculate(self):
        '''
        Calculate the IC scores
        '''
        ic_scores =  {'Total'           : 0,
                      'charged-charged' : 0, 
                      'charged-polar'   : 0,
                      'charged-apolar'  : 0,
                      'polar-polar'     : 0,
                      'polar-apolar'    : 0,
                      'apolar-apolar'   : 0 }
        for x in xrange(len(self.contact_distances)):
            res_type1 = self.contact_pos1[x].get_type()
            res_type2 = self.contact_pos2[x].get_type()
            if ic_scores.has_key(self.res_types[res_type1]+'-'+self.res_types[res_type2]):
                ic_scores[self.res_types[res_type1]+'-'+self.res_types[res_type2]] += 1
                ic_scores['Total'] += 1
            elif ic_scores.has_key(self.res_types[res_type2]+'-'+self.res_types[res_type1]):
                ic_scores[self.res_types[res_type2]+'-'+self.res_types[res_type1]] += 1
                ic_scores['Total'] += 1
        return ic_scores

class Elec_potential(object):
    '''
    Compute the electrostatic energy using coulomb's law (q1*q2/r**2), asuming each amino acid
    has the same charge.
    The more negative, more energy.
    '''
    def __init__(self, interaction):
        '''
        Contructor
        '''
        self.interaction = interaction
        interacting_residues = self.interaction.get_interacting_residues(c_type = 'CB', max_distance = 40, uniq=False)
        self.contact_pos1 = interacting_residues[0]
        self.contact_pos2 = interacting_residues[1]
        self.contact_distances = interacting_residues[2]

    def calculate(self):
        '''
        Calculate the absolute electrostatic energy
        '''
        Elec_energy = 0
        for x in xrange(len(self.contact_distances)):
            q1 = self.contact_pos1[x].get_charge()
            q2 = self.contact_pos2[x].get_charge()
            r2  = self.contact_distances[x]**2
            Elec_energy += (q1*q2)/r2
        return Elec_energy

    def calculate_normalized(self, randoms = 100):
        '''
        Calculate the normalized electrostatic energy
        '''
        Elec_distribution = []
        for x in xrange(randoms):
            Elec_energy = 0
            for x in xrange(len(self.contact_distances)):
                random_pos1 = random.choice(self.interaction.get_structure1().get_residues())
                random_pos2 = random.choice(self.interaction.get_structure2().get_residues())
                q1 = random_pos1.get_charge()
                q2 = random_pos2.get_charge()
                r2  = self.contact_distances[x]**2
                Elec_energy += (q1*q2)/r2
            Elec_distribution.append(Elec_energy)
        ZElec_energy = (self.calculate()-np.mean(Elec_distribution))/np.std(Elec_distribution)
        return ZElec_energy

class Polarity_potential(object):
    '''
    Compute a a/polar score based on the polarity of the contact residues.
    '''
    def __init__(self, interaction):
        '''
        Contructor
        '''
        self.interaction = interaction
        interacting_residues = self.interaction.get_interacting_residues(c_type = 'CB', max_distance = 12, uniq=False)
        self.contact_pos1 = interacting_residues[0]
        self.contact_pos2 = interacting_residues[1]
        self.contact_distances = interacting_residues[2]

    def calculate(self):
        '''
        Calculate the absolute potential
        '''
        Polar_score = 0
        Apolar_score = 0
        for x in xrange(len(self.contact_distances)):
            if self.contact_pos1[x].is_polar() and self.contact_pos2[x].is_polar():
                Polar_score += 1
            if not self.contact_pos1[x].is_polar() and not self.contact_pos2[x].is_polar():
                Apolar_score += 1
        return Polar_score, Apolar_score

    def calculate_normalized(self, randoms = 100):
        '''
        Calculate the normalized potential
        '''
        Polar_distribution = []
        Apolar_distribution = []
        for x in xrange(randoms):
            Polar_score = 0
            Apolar_score = 0
            for x in xrange(len(self.contact_distances)):
                random_pos1 = random.choice(self.interaction.get_structure1().get_residues())
                random_pos2 = random.choice(self.interaction.get_structure2().get_residues())
                if self.random_pos1.is_polar() and self.random_pos2.is_polar():
                    Polar_score += 1
                if not self.random_pos1.is_polar() and not self.random_pos2.is_polar():
                    Apolar_score += 1
            Polar_distribution.append(Polar_score)
            Apolar_distribution.append(Apolar_score)
        ZPolar_score = (self.calculate()[0]-np.mean(Polar_distribution))/np.std(Polar_distribution)
        ZApolar_score = (self.calculate()[1]-np.mean(Apolar_distribution))/np.std(Apolar_distribution)
        return ZPolar_score, ZApolar_score



"""
Nucleotide

author: jbonet
date:   02/2013

@oliva's lab
"""

from .          import Residue
from SBI.data   import nucleic3to1, nucleic1to3
import numpy    as np

class ResidueOfNucleotide(Residue):
    """
    A {Nucleotide} collects a series of {NucleotideAtom}s
    """
    def __init__(self, number, version, Rtype, mode):
	"""
        @type  number: Integer
        @param number: Residue number

        @type  version: Char
        @param version: Optional char used on pdbs to change count

        @type  type: String
        @param type: Residue type

        @type  mode: String
        @param mode: Residue mode: ATOM or HETATM
        """
        super(ResidueOfNucleotide, self).__init__(number = number, version = version, Rtype = Rtype, mode = mode)
        
        self._p        = None
        self._o3       = None

    #
    # ATTRIBUTES
    #
    @property
    def p(self):
        """
        Returns the Phosphate {NucleotideAtom}
        @rtype: {NucleotideAtom}
        """
        return self._p

    @property
    def o3(self):
        """
        Returns the Sugar O3' {NucleotideAtom}
        @rtype: {NucleotideAtom}
        """
        return self._o3

    @property
    def backbone(self):
        """
        Returns a list of the backbone {NucleotideAtom}s
        @rtype: List of {NucleotideAtom}s
        """
        return self._backbone_atoms

    @property
    def single_letter(self):
        """
        Returns the AminoAcid identifier as a single letter code
        @rtype: String
        """
        return nucleic3to1[self.type]

    @property
    def standard_type(self):
        """
        For some Nucleotide are HETATM, we can return the standard Nucleotide
        @rtype: String
        """
        return nucleic1to3[self.single_letter]

    @property
    def nitrogenous_base(self):
        """
        Returns the Nucleotide base as a single letter code
        @rtype: String
        """
        return nitrogenous_bases[self.single_letter]

    #
    # BOOLEANS
    #
    @property
    def has_p(self):
        """
        Checks if the Nucleotide has Phosphate
        @rtype: Boolean
        """
        return self._p is not None

    @property
    def has_o3(self):
        """
        Checks if the Nucleotide has Sugar O3'
        @rtype: Boolean
        """
        return self._o3 is not None

    #
    # METHODS
    #
    def normalize(self):
        self._type = self.standard_type
        self._mode = 'ATOM'
        
    def remove_side_chain(self):
        # self._atoms = self._backbone_atoms
        self._sidechain_atoms = []
        self._sidechain_coordinates = None

    def _distance_p(self, nucleotide):
        """
        Calculates the p-p distance between two residues

        @type  nucleotide: {Nucleotide}
        @param nucleotide: Nucleotide to which we want to find the distance

        @rtype: {Contact}
        @rtype: None if one of them does not have p
        """
        if not self.has_p or not nucleotide.has_p:
            return (None, None, -1)

        return (self.p, nucleotide.p, self.p.distance(atom = nucleotide.p))

    #
    # OVERRIDE PARENT'S METHODS
    #
    def add_atom(self, atom): 
        """
        Adds a new {NucleotideAtom} to the {Nucleotide}
        This includes:
            filling the all_coordinates attribute (parent)
            assign P, BB and backbone
        @type  atom: {AminoAtom}
        @param atom: New {AminoAtom} added to the {AminoAcid}
        """
        super(ResidueOfNucleotide, self).add_atom(atom = atom)

        if atom.is_Phosphate:
            self._p = atom

        if atom.is_SugarOxygen3:
            self._o3 = atom

    def distance(self, residue, dist_type):
        """
        Calculates the distance between two residues

        @type  residue: {Nucleotide}
        @param residue: Nucleotide to which we want to find the distance

        @type  dist_type: String
        @param dist_type: Type of distance to evaluate
        @limit dist_type: Accepted: distance.types (p, min, geometric)

        @rtype: (AT1, AT2, DISTANCE)
        """
        if not isinstance(residue, ResidueOfNucleotide):
            if isinstance(residue, ResidueOfAminoAcid):
                return super(ResidueOfNucleotide, self).distance(residue = residue, dist_type = dist_type)
            else:
                return super(ResidueOfAminoAcid, self).distance(residue = residue, dist_type = dist_type)
        if dist_type.strip().lower() == 'p':
            return self._distance_p(nucleotide = residue)
        else:
            return super(ResidueOfNucleotide, self).distance(residue = residue, dist_type = dist_type)


    def follows(self, residue):
        return residue.is_followed(self)

    def is_followed(self, residue):
        p0, o30 = self.p,    self.o3
        p1, o31 = residue.p, residue.o3
        a, b, distance = self.distance(residue, "geometric")

        if p0 is not None and p1 is not None and o30 is not None and o31 is not None:
            return o30.distance(p1) < o31.distance(p0) and distance <= 7.0 # measured distance P-P is around 7.0
        else:
            return self._backbone_atoms[-1].distance(residue._backbone_atoms[0]) < residue._backbone_atoms[-1].distance(self._backbone_atoms[0]) and distance <= 7.0 # measured distance P-P is around 7.0



"""
DSSP

author: jbonet
date:   03/2013

@oliva's lab
"""

global exposed_threshold
exposed_threshold = 2.5 #Why? DSSP exposition refers only to secondary chain (aprox.) see PMID: 15768403

from SBI.data import aminoacids3to1, aminoacids_surface

class DSSP(object):
    """
    The {DSSP} stores the dssp prediction
    """
    def __init__(self, secondary_structure, accessibility, AAtype):

        self._ss         = secondary_structure
        self._access     = int(accessibility)
        self._type       = AAtype
        if AAtype in aminoacids_surface:
            self._access10   = int((10 * self._access) / aminoacids_surface[AAtype])
            self._accesscode = self._codifyaccess((float(self._access)/ aminoacids_surface[AAtype]) * 100)
        else:
            self._access10   = 1
            self._accesscode = 1
        self._exposed    = self._access10 > exposed_threshold
        self._rnhoa      = None
        self._enhoa      = None
        self._rohna      = None
        self._eohna      = None
        self._rnhob      = None
        self._enhob      = None
        self._rohnb      = None
        self._eohnb      = None

    #
    # ATTRIBUTES
    #
    @property
    def secondary_structure(self): return self._ss

    @property
    def aminoacid(self):            return self._type

    @property
    def accessibility(self):       return self._access

    @property
    def accessibility10(self):     return self._access10

    @property
    def accesscode(self):          return str(self._accesscode)

    @staticmethod
    @property
    def exposition_threshold():
        global exposed_threshold
        return exposed_threshold

    @property
    def exposed(self):             return self._exposed

    @property
    def nhoa(self):
        return self._rnhoa, self._enhoa

    @property
    def ohna(self):
        return self._rohna, self._eohna

    @property
    def nhob(self):
        return self._rnhob, self._enhob

    @property
    def ohnb(self):
        return self._rohnb, self._eohnb

    #
    # FUNCTIONS
    #
    def add_hydrogen_links(self, nhoa, ohna, nhob, ohnb):
        self._rnhoa = int(  nhoa.split(',')[0].strip())
        self._enhoa = float(nhoa.split(',')[1].strip())
        self._rohna = int(  ohna.split(',')[0].strip())
        self._eohna = float(ohna.split(',')[1].strip())
        self._rnhob = int(  nhob.split(',')[0].strip())
        self._enhob = float(nhob.split(',')[1].strip())
        self._rohnb = int(  ohnb.split(',')[0].strip())
        self._eohnb = float(ohnb.split(',')[1].strip())

    def _codifyaccess(self, value):
        value = float(value)
        if value == 0:
            return '*'
        elif value > 100:
            return '?'
        else:
            if   value > 0  and value <= 10:  return '1'
            elif value > 10 and value <= 20:  return '2'
            elif value > 20 and value <= 30:  return '3'
            elif value > 30 and value <= 40:  return '4'
            elif value > 40 and value <= 50:  return '5'
            elif value > 50 and value <= 60:  return '6'
            elif value > 60 and value <= 70:  return '7'
            elif value > 70 and value <= 80:  return '8'
            elif value > 80 and value <= 90:  return '9'
            elif value > 90 and value <= 100: return '#'




"""
NucleotideChain

author: jbonet
date:   02/2013

@oliva's lab
"""

from ..atom       import Atom,    AtomOfNucleotide
from ..residue    import Residue, ResidueOfNucleotide
from .            import Chain
from SBI.data import nucleic3to1

class ChainOfNucleotide(Chain):
    """
    A {NucleotideChain} represents a collection of {AminoAcids}s
    """
    chaintype   = 'N'
    atomtype    = AtomOfNucleotide
    resitype    = ResidueOfNucleotide
    dictitype   = nucleic3to1
    
    def __init__(self, pdb, chain):
        """
        @type  pdb: String
        @param pdb: PDB identifier

        @type  chain: String
        @param chain: Chain identifier
        """
        super (ChainOfNucleotide, self).__init__(pdb = pdb, chain = chain)

    #
    # ATTRIBUTES
    #
    @property
    def nucleotides(self):
        """
        Returns the list of {AminoAcid}s
        @rtype: List of {AminoAcid}s
        """
        return self._structure

    @property
    def first_nucleotide(self):
        """
        Returns the first {AminoAcid}
        @rtype: {AminoAcid}
        """
        return self.nucleotides[0]

    @property
    def last_nucleotide(self):
        """
        Returns the last {AminoAcid}
        @rtype: {AminoAcid}
        """
        return self.nucleotides[-1]

    #
    # METHODS
    #
    def nucleotide_sequence(self):
        """
        Returns the exact sequence in the crystal
        @rtype: String
        """
        seq = ''
        for nc in self.nucleotides:
            seq += nc.single_letter
        return seq

    def gapped_nucleotide_sequence(self):
        """
        Returns the sequence of the crystal with 'X' where
        the crystal contains gaps.
        @rtype: String
        """
        seq = ''
        pos = self.nucleotides[0].number - 1
        for nc in self.nucleotides:
            if nc.number == pos + 1:
                seq += nc.single_letter
            else:
                for x in range(nc.number - pos - 1):
                    seq += 'x'
                seq += nc.single_letter
            pos = nc.number
        return seq

    def nucleotide_idx(self):
        """
        Returns indexes (number + version) of the residues separated by x on gaps
        @rtype: String
        """
        return self._chain_idx()

    def full_nucleotide_sequence(self):
        """
        Returns the sequence of the crystal with 'X' where
        the crystal contains gaps AND before the sequence if it does not start at 1.
        @rtype: String
        """
        if self.nucleotides[0].number <= 1: return self.gapped_nucleotide_sequence()

        seq = ''
        for x in range(self.nucleotides[0].number - 1): seq += 'x'
        return seq + self.gapped_nucleotide_sequence



import sys
import argparse
import os
import re
import ConfigParser
import shutil
import subprocess
import time
import random
from collections import Counter

# Add '.' to sys.path
src_path = os.path.abspath(os.path.dirname(__file__))
sys.path.append(src_path)

# Read configuration file
config = ConfigParser.ConfigParser()
config_file = os.path.join(src_path, 'config.ini')
config.read(config_file)

# Add SBI library path to sys.path
src_path = os.path.join(config.get('Paths','modppi_path'),config.get('Paths','sbi_library_path'))
sys.path.append(src_path)

# Add functions path to sys.path
src_path = os.path.join(config.get('Paths','modppi_path'),config.get('Paths','functions_path'))
sys.path.append(src_path)

# Import functions
from SeqIO import *
from functions import *
from SBI.external.blast import blast_parser
from SBI.structure.contacts import Complex
from SBI.sequence import Sequence
from SBI.structure import PDB
from FilterHomologPairs import filter_homologs
from Bio import SeqIO
from Bio import ExPASy
from Bio import AlignIO
from Bio.Align.Applications import *
import BioLib


def main():
    
    #initialize
    options    = parse_user_arguments()
    verbose    = options.show
    zrank      = options.zrank
    foldx      = options.foldx
    ssp        = options.ssp
    rosetta    = options.rosetta
    input_list = os.path.join(options.ppi_list)
    output_dir = options.outdir
    hbplus     = options.hbplus
    label      = options.label
    boxplot    = options.boxplot

    #Check that at least one method to calculate the energy is set
    if not zrank and not foldx and not ssp and not rosetta:
       ssp=True

    if options.ppi_list is None:
      sys.stderr.write("ERROR: Missing argument input list '-i'\n")
      exit(0)
    if not fileExist(input_list):
      sys.stderr.write("ERROR: Missing file %s\n"%input_list)
      exit(0)
    if not os.path.exists(options.dummy_dir): os.makedirs(options.dummy_dir)
    dummy_dir=os.path.join(options.dummy_dir,"analysis")
    if not os.path.exists(dummy_dir): os.makedirs(dummy_dir)
    #Write  distributions for boxplot
    if options.outdir is None:
        output_dir=os.path.abspath("analysis")
    else:
        output_dir=options.outdir
    if not os.path.exists(output_dir):os.makedirs(output_dir)
         

    #Create a parsed list of models (modify their names in case hydrogens need to be added)
    # information from parser_list_of_models is a dictionary:
    # key= 2-tuple (pair1, pair2) with names of the interactors
    # value= list [] with 2-tuples of (name(number),name(file_list)) of the cluster name(pose) and the name of the file with the list of PDB files
    list_of_models=parser_list_of_models(input_list, options)

    #Check the sequences of the pairs and the differences of the interface for each cluster-pose
    # information from compare_interface_sequences is a dictionary:
    # key= (cluster-pose name,(pair1,pair2)) 2-tuple (pair1,pair2) are names of interactors
    # value=  2-tuples of:
    #    2-tuple (ratio1,ratio2) with ratios of structures where a difference in sequence with respect to its wild-type is involved in the interface
    #
    pose_interfaces=compare_interface_sequences(list_of_models,dummy_dir)
    

    binding={}

    #Run zrank on folders if active option zrank
    if zrank:
     if verbose: sys.stdout.write("\t-- Calculate ZRANK...\n")
     try:
       binding_by_zrank= calculate_zrank(list_of_models,dummy_dir,label,verbose)
     except Exception as e:
       sys.stderr.write("FAIL %s\n"%(e))
       exit(0)
     for key,value in binding_by_zrank.iteritems():
         binding.setdefault(key,value)

    #Run foldx on folders if active option foldx
    if foldx:
     if verbose: sys.stdout.write("\t-- Calculate FOLDX...\n")
     try:
       binding_by_foldx= calculate_foldx(list_of_models,dummy_dir,label,verbose)
     except Exception as e:
       sys.stderr.write("FAIL %s\n"%(e))
       exit(0)
     for key,value in binding_by_foldx.iteritems():
         binding.setdefault(key,value)

    #Run SSP on folders if active option ssp
    if ssp:
     if verbose: sys.stdout.write("\t-- Calculate by Split Potentials...\n")
     try:
       binding_by_ssp= calculate_split_potentials(list_of_models,dummy_dir,label,verbose)
     except Exception as e:
       sys.stderr.write("FAIL %s\n"%(e))
       exit(0)
     for key,value in binding_by_ssp.iteritems():
         binding.setdefault(key,value)

    #Run Rosetta on folders (by default is always done)
    if rosetta:
       if verbose: sys.stdout.write("\t-- Calculate ddG with Rosetta...\n")
       try:
         binding_by_rosetta=calculate_rosetta(list_of_models,dummy_dir,label,verbose)
       except Exception as e:
         sys.stderr.write("FAIL %s\n"%(e))
         exit(0)
       for key,value in binding_by_rosetta.iteritems():
           binding.setdefault(key,value)
   
    #Prepare Plots
    energy_type=set()
    pair_list=set()
    pose_list=set()
    plot_list=set()
    for ((a,b),pose,key),outfile  in binding.iteritems():
        energy_type.add(key)
        pair_list.add((a,b))
        pose_list.add(pose)
    for key in energy_type:
        output=os.path.join(output_dir,label+"."+key+"_distribution.dat") 
        name_file=os.path.join(output_dir,label+"."+key+"_distribution")
        fo=open( output,"w")
        for pose in pose_list:
         if pose == 0:
           output_pose=os.path.join(output_dir,label+"."+key+".all_distribution.dat") 
           name_file_pose=os.path.join(output_dir,label+"."+key+".all_distribution")
         else:
           output_pose=os.path.join(output_dir,label+"."+key+".cluster_"+str(pose)+"_distribution.dat") 
           name_file_pose=os.path.join(output_dir,label+"."+key+".cluster_"+str(pose)+"_distribution")
         fp=open( output_pose,"w")
         for a,b in pair_list:
           if binding.has_key(((a,b),pose,key)):
            outfile= binding[((a,b),pose,key)]
            if pose_interfaces.has_key((pose,(a,b))):
             fp.write("%s %s\n"%(a+"-"+b+"#"+str(pose)+interface_affected(pose_interfaces[pose,(a,b)]),outfile))
         fp.close()
         plot_list.add((key,name_file_pose,1,len(pair_list))) 
        ncmax=0
        for a,b in pair_list:
          nc=0
          for pose in pose_list:
            if pose==0:continue
            if binding.has_key(((a,b),pose,key)):
             nc=nc+1
             if nc>ncmax:ncmax=nc
             outfile= binding[((a,b),pose,key)]
             if pose_interfaces.has_key((pose,(a,b))):
              fo.write("%s %s\n"%(a+"-"+b+"#"+str(pose)+interface_affected(pose_interfaces[pose,(a,b)]),outfile))
        fo.close()
        plot_list.add((key,name_file,ncmax,ncmax*len(pair_list)))

    #Automatic BOXPLOTS
    if boxplot:
       boxplot_path = os.path.join(config.get('Paths','modppi_path'),config.get('Paths','functions_path'))
       boxplot_exe  = os.path.join(boxplot_path,"boxplot.py")
       python_path  = config.get('Paths', 'python_path')
       python_exe   = os.path.join(python_path,"python")
       for key,name,nc,nh in plot_list:
         input_list  = name + ".dat"
         boxplot_name= name
         output_name = name + "_statistic.out"
         os.system('%s %s -l %s -o %s -nc %d -nh %d -t "Distribution of %s" >& %s'%(python_exe,boxplot_exe,input_list,boxplot_name,nc,nh,key,output_name))
         if verbose: sys.stdout.write('\t-- boxplot SCORE %s INPUT %s OUTPUT %s STATISTICS %s\n'%(key,input_list,boxplot_name,output_name))

    #Remove dummy folders
    if not verbose: shutil.rmtree(dummy_dir)


def calculate_zrank(list_of_models,dummy_dir,label,verbose):

  #Initialize
  zrank_exe  = config.get('Paths','zrank_path')
  binding={}

  for pair,folders in list_of_models.iteritems():
    for pose,folder in folders:
       #if verbose: sys.stdout.write("\t\t-- Use list %s for %s...\n"%(folder.split("/")[-1],pair))
       if verbose: sys.stdout.write("\t\t-- Use list %s for %s...\n"%(folder,pair))
       output_zrank=folder+".zr.out"
       output_zrank_label=folder+".zrank."+label+".out"
       output_zrank_failed=folder+".zrank."+label+".fail"
       if not fileExist(output_zrank_label):
        try:
         #process=subprocess.Popen([zrank_exe,folder], stderr = subprocess.STDOUT)
         os.system("%s %s"%(zrank_exe,folder))
        except Exception as e:
         sys.stderr.write("ERROR: %s\n"%e)
         dummy_file=os.path.join(dummy_dir,folder.split("/")[-1])
         dummy_list=open(os.path.join(dummy_dir,folder.split("/")[-1]),"w")
         if verbose: sys.stdout.write("\t\t\t-- Create new list for %s (%s)...\n"%(str(pair),dummy_file))
         path="/".join(folder.split("/")[:-1])
         for models_ready in os.listdir(path):
           if models_ready.strip().endswith(".pdb"):
              dummy_list.write("%s\n"%os.path.join(path,models_ready.strip()))
         dummy_list.close()
         if fileExist(output_zrank): shutil.move(output_zrank,output_zrank_failed)
         shutil.move(dummy_file,folder)
        if not fileExist(output_zrank):
         try:
          #process=subprocess.Popen([zrank_exe,folder], stderr = subprocess.STDOUT)
          #process.wait()
          os.system("%s %s"%(zrank_exe,folder))
         except Exception as e:
          sys.stderr.write("ERROR: %s\n"%e)
        if fileExist(output_zrank):
         check_file=open(folder,"r")
         check_zrank=open(output_zrank,"r")
         check_file_set=set()
         check_zrank_set=set()
         for models_ready in check_file:
             check_file_set.add(models_ready.strip())
         for models_ready in check_zrank:
             check_zrank_set.add(models_ready.strip().split()[0])
         check_file.close()
         check_zrank.close()
         if check_file_set!=check_zrank_set:
            sys.stderr.write("ERROR: ZRANK failed for some models\n")
            if verbose: sys.stdout.write("\t\t\t-- Create new list for %s...\n"%(str(pair)))
            dummy_file=os.path.join(dummy_dir,folder.split("/")[-1])
            dummy_list=open(os.path.join(dummy_dir,folder.split("/")[-1]),"w")
            path="/".join(folder.split("/")[:-1])
            for models_ready in os.listdir(path):
              if models_ready.strip().endswith(".pdb"):
                 dummy_list.write("%s\n"%(os.path.join(path,models_ready.strip())))
            dummy_list.close()
            shutil.move(output_zrank,output_zrank_failed)
            shutil.move(dummy_file,folder)
        if not fileExist(output_zrank):
         try:
          #process=subprocess.Popen([zrank_exe,folder], stderr = subprocess.STDOUT)
          #process.wait()
          os.system("%s %s"%(zrank_exe,folder))
         except Exception as e:
          sys.stderr.write("ERROR: %s\n"%e)
        if not fileExist(output_zrank):
         sys.stderr.write("ERROR: %s doesn't exist\n"%output_zrank)
         sys.stderr.write("SKIP:  %s\n"%output_zrank_label)
        else:
         shutil.move(output_zrank,output_zrank_label)
       #When finally there is a result for zrank added
       binding.setdefault((pair,pose,"zrank"),output_zrank_label)

  return binding


def calculate_foldx(list_of_models,dummy_dir,label,verbose):

  #Initialize
  foldx_path  = config.get('Paths','foldx_path')
  foldx_exe   = os.path.join(foldx_path,"foldx")
  rotabase    = os.path.join(foldx_path,"rotabase.txt")
  binding={}

  for pair,folders in list_of_models.iteritems():
    for pose,folder in folders:
       #if verbose: sys.stdout.write("\t\t-- Use list %s for %s...\n"%(folder.split("/")[-1],pair))
       if verbose: sys.stdout.write("\t\t-- Use list %s for %s...\n"%(folder,pair))
       output_foldx=folder+".foldx."+label+".out"
       if not fileExist(output_foldx):
         output_dummy=os.path.basename(folder)+".foldx."+label+".out"
         foldx_list=[]
         fo=open(folder,"r")
         for line in fo:
           p=line.strip()
           dummy_file=os.path.join(dummy_dir,os.path.basename(p))
           shutil.copyfile(p,dummy_file)
           foldx_list.append(p)
         fo.close()
         cwd = os.getcwd()
         os.chdir(dummy_dir)
         energy={}
         for pdb_check in foldx_list:
           pdb_name=os.path.basename(pdb_check)
           pdb_root="".join(pdb_name.split(".")[0:-1])
           if not fileExist(pdb_name):
              raise ValueError("Missing file %s"%(pdb_name))
           # execute foldx optimization
           pdb_repair=pdb_root+"_Repair.pdb"
           pdb_repair_home = os.path.join(os.path.dirname(pdb_check),pdb_repair)
           if fileExist(pdb_repair_home):
              shutil.copyfile(pdb_repair_home,pdb_repair)
           else:
              os.system("%s  --rotabaseLocation %s --repair_Interface ONLY  --command RepairPDB --pdb=%s >& %s"%(foldx_exe,rotabase,pdb_name,pdb_root+"_Repair.log"))
              shutil.copyfile(pdb_repair,pdb_repair_home)
           # execute foldx AnalyseComplex
           if not fileExist(pdb_repair):
              raise ValueError("Missing file %s"%(pdb_repair))
           os.system("%s  --rotabaseLocation %s --command AnalyseComplex --analyseComplexChains=A,B --pdb=%s >& %s"%(foldx_exe,rotabase,pdb_repair,pdb_root+"_AC.log"))
           #parse foldx output
           pdb_fxout="Interaction_"+pdb_root+"_Repair_AC.fxout"
           if verbose: sys.stdout.write("\t\t\t-- Open %s ...\n"%(pdb_fxout))
           if fileExist(pdb_fxout):
              fo=open(pdb_fxout,"r")
              for line in fo:
                  word=line.strip().split()
                  if len(word)>0:
                   if word[0].lstrip("./") == pdb_repair:
                     energy.setdefault(pdb_check,float(word[5]))
              fo.close()
           else:
              raise ValueError("Missing file %s"%(pdb_fxout))
         if verbose: sys.stdout.write("\t\t\t-- Use FoldX output %s ...\n"%(output_dummy))
         fo=open(output_dummy,"w")
         for pdb,ene in energy.iteritems():
           fo.write("%s\t%s\n"%(pdb,ene))
         fo.close()
         shutil.move(output_dummy,output_foldx)
         os.chdir(cwd)
       else:
         if verbose: sys.stdout.write("\t\t\t-- Use FoldX output %s ...\n"%(os.path.basename(output_foldx)))
       #Add result of foldx
       binding.setdefault((pair,pose,"foldx"),output_foldx)

  return binding

class SPPPI(object):
    '''
    Analyze a protein folds using the split potentials and prints an XML file
    '''
    def __init__(self, receptor, ligand, pot_type):
        '''
        Contructor
        '''
        self.receptor    = receptor
        self.ligand      = ligand
        self.pot_type    = pot_type

 
    def get_energies(self):
        '''
        Compute the split potentials for a ppi
        '''
        receptor=self.receptor
        ligand  =self.ligand

        receptor.set_dssp()
        receptor.normalize_residues()
        ligand.set_dssp()
        ligand.normalize_residues()

        ppi = BioLib.Interaction(receptor, ligand)

        # Compute split potentials
        if self.pot_type == 'CB':
            cutoff = 12
        else: 
            cutoff = 5

        split_potentials = BioLib.SplitPotentialsPPI(c_type=self.pot_type, cutoff=cutoff) # definition of SplitPotentialsPPI class (BioLib.Docking)
        global_energies = split_potentials.calculate_global_energies(ppi, Zscores=True) # calculation of global energies using the method of the SplitPotentialsPPI class

        return global_energies



def calculate_split_potentials(list_of_models,dummy_dir,label,verbose):

  #Initialize
  ssp_list = config.get('Parameters','ssp_score')
  ssp_type = config.get('Parameters','ssp_type')

  ssp_score_list=ssp_list.split(";")

  binding={}

  for pair,folders in list_of_models.iteritems():
   for pose,folder in folders:
    for ssp in ssp_score_list:
       ssp_score=ssp.replace(" ","")
       #if verbose: sys.stdout.write("\t\t-- Use list %s for %s...\n"%(folder.split("/")[-1],pair))
       if verbose: sys.stdout.write("\t\t-- Use list %s for %s...\n"%(folder,pair))
       output_ssp=folder+"."+ssp_score+"."+label+".out"
       if not fileExist(output_ssp):
         output_dummy=os.path.basename(folder)+"."+ssp_score+"."+label+".out"
         ssp_list=[]
         fo=open(folder,"r")
         for line in fo:
           p=line.strip()
           dummy_file=os.path.join(dummy_dir,os.path.basename(p))
           shutil.copyfile(p,dummy_file)
           ssp_list.append(p)
         fo.close()
         cwd = os.getcwd()
         os.chdir(dummy_dir)
         energy={}
         for pdb_check in ssp_list:
           pdb_name=os.path.basename(pdb_check)
           pdb_root="".join(pdb_name.split(".")[0:-1])
           if not fileExist(pdb_name):
              raise ValueError("Missing file %s"%(pdb_name))
           # execute ssp 
           pdb_ppi=BioLib.PDB.read_pdb(pdb_name)
           receptor     = pdb_ppi.pop()
           ligand       = pdb_ppi.pop()
           ssp_ppi      = SPPPI(receptor,ligand,ssp_type)
           ssp_energies = ssp_ppi.get_energies()
           zscore=0
           if ssp_score.lower().startswith("z"):   zscore=1
           if ssp_score.lower().endswith("local"): ssp_ene="D-LOCAL"
           if ssp_score.lower().endswith("pair"):  ssp_ene="D-PAIR"
           if ssp_score.lower().endswith("comb"):  ssp_ene="D-COMB"
           if ssp_score.lower().endswith("s3dc"):  ssp_ene="D-S3DC"
           energy.setdefault(pdb_check,float(ssp_energies[zscore][ssp_ene]))
         if verbose: sys.stdout.write("\t\t\t-- Use Split Potentials output %s ...\n"%(output_dummy))
         fo=open(output_dummy,"w")
         for pdb,ene in energy.iteritems():
           fo.write("%s\t%s\n"%(pdb,ene))
         fo.close()
         shutil.move(output_dummy,output_ssp)
         os.chdir(cwd)
       else:
         if verbose: sys.stdout.write("\t\t\t-- Use Split Potentials output %s ...\n"%(os.path.basename(output_ssp)))
       #Add result of ssp
       binding.setdefault((pair,pose,ssp_score),output_ssp)

  return binding

def calculate_rosetta(list_of_models,dummy_dir,label,verbose):

    binding={}

    if verbose: sys.stdout.write("\t\t-- Cleaning output files...\n")
    for pair,folders in list_of_models.iteritems():
     for pose,folder in folders:
      path_folder="/".join(folder.split("/")[:-1])
      clean_file =folder.split("/")[-1]
      if verbose: sys.stdout.write("\t\t\t Clean directory %s \n"%path_folder)
      for files in os.listdir(path_folder):
        m=re.search("^%s"%clean_file,files)
        if m:
          line=files.lstrip(folder).rstrip().split(".")
          if line[-1]=="out":
            if len(line[0].split("_")) > 1:
             check=line[0].split("_")[1]
             if check == "mean" or check == "all":
                if verbose: sys.stdout.write("\t\t\t\t Remove file %s \n"%files)
                os.remove(os.path.join(path_folder,files))
    for pair,folders in list_of_models.iteritems():
     for pose,folder in folders:
      if verbose: sys.stdout.write("\t\t-- Use list %s for %s...\n"%(folder.split("/")[-1],pair))
      fd=open(folder,"r")
      for line in fd:
        pdb_file=line.strip().split()[0]
        energy_file=pdb_file+"."+"interface_analyzer"+"."+label+".out"
        if verbose: sys.stdout.write("\t\t\t-- Interface Analyzer output: %s\n"%energy_file.split("/")[-1])
        try:
          binding_data=binding_energy(pdb_file,energy_file,dummy_dir)
          energy_type=set()
          for key,score in binding_data.iteritems():
            feature,chain_A,chain_B=key
            feature_all=feature+"_all"
            out_feature_all=folder+"."+feature+"_all."+label+".out"
            fa=open(out_feature_all,"a")
            fa.write("%s\t%f\n"%(pdb_file+"_"+chain_A+chain_B,score))
            fa.close()
            energy_type.add(feature)
          for feature in energy_type:
            feature_mean=feature+"_mean"
            out_feature_mean=folder+"."+feature+"_mean."+label+".out"
            fm=open(out_feature_mean,"a")
            dat=[score for (e,a,b),score in binding_data.iteritems() if feature == e]
            if len(dat)>0:
              mean=sum(dat)/len(dat)
              fm.write("%s\t%f\n"%(pdb_file,mean))
            fm.close()
        except ValueError as e:
          sys.stderr.write("ERROR %s\n"%e)
      fd.close()
      for feature in energy_type:
          binding.setdefault((pair,pose,feature+"_all"),folder+"."+feature+"_all."+label+".out")
          binding.setdefault((pair,pose,feature+"_mean"),folder+"."+feature+"_mean."+label+".out")

    return binding

              
def compare_templates(sa,sb):

   new_sa=set()
   new_sb=set()
   for p in sa:
     a,b = p.rstrip(".pdb").split("::")
     template_a="_".join(a.split("_")[:-1])
     template_b="_".join(b.split("_")[:-1])
     new_sa.add((template_a,template_b))
   for p in sb:
     a,b = p.rstrip(".pdb").split("::")
     template_a="_".join(a.split("_")[:-1])
     template_b="_".join(b.split("_")[:-1])
     new_sb.add((template_a,template_b))


   return (new_sa,new_sb)
          
def binding_energy(pdb_file,energy_file,dummy_dir="/tmp"):
   ''' 
    Binding_energy function
    uses a pdb_file name as input and the name energy_file to write the output of energies
    The function returns a dictionary:
     key= 3-tuple (a,b,c) of
          a=the type of energy score from Rosetta: ddG, dGx, dSASA...
          b=chain name of one partner,A
          c=chain name of the interactor,B
     value=score
   '''
   #Initialize
   rosetta=config.get('Paths','rosetta_path')
   interface_analyzer=config.get('Paths','interface_analyzer')
   database=os.path.join(rosetta,'database')
   binding={}
   if fileExist(pdb_file):
     pdb=PDB(pdb_file) 
   else:
     raise ValueError("Missing file %s"%(pdb_file))
   chains=[x for x in pdb.chain_identifiers]
   for x in chains:
    for y in chains:
     if x==y:continue
     output=energy_file+"_"+x+"_"+y
     commands=[]
     commands.append("%s"%interface_analyzer)
     commands.append("-pack_input")
     commands.append("-pack_separated")
     commands.append("-ignore_zero_occupancy=False")
     commands.append("-in:ignore_unrecognized_res")
     commands.append("-add_regular_scores_to_scorefile")
     commands.append("-run:constant_seed")
     commands.append("-nodelay")
     commands.append("-database %s"%(database))
     commands.append("-s %s"%(pdb_file))
     commands.append("-out:file:score_only %s"%(output))
     commands.append("-fixedchains %s %s"%(x,y))
     if not fileExist(output):
        #print " ".join(commands)
        os.system("%s >& %s"%(" ".join(commands),os.path.join(dummy_dir,"interface_analyzer_"+output.split("/")[-1]+".log")))
        #process=subprocess.Popen(commands, stderr = subprocess.STDOUT, stdout = subprocess.STDOUT)
        #process.wait()
     try:
      f=open(output,"r")
      for line in f:
       word=line.strip().split()
       if word[0]=="SCORE:" and word[1]!="total_score":
        dGx   =float(word[3])
        ddG   =float(word[5])
        dSASAh=float(word[7])
        dSASA =float(word[8])
        dSASAp=float(word[9])
      f.close()
      binding.setdefault(("dGx",x,y),dGx)
      binding.setdefault(("ddG",x,y),ddG)
      binding.setdefault(("dSASAh",x,y),dSASAh)
      binding.setdefault(("dSASA",x,y),dSASA)
      binding.setdefault(("dSASAp",x,y),dSASAp)
     except:
      sys.stderr.write("ERROR: Rosetta FAIL to obtain %s \n"%(output))

   return binding


def parser_list_of_models(input_list,options):

    renumerate=options.renumerate
    fasta_file=options.fasta_file
    dummy_dir=options.dummy_dir
    verbose  =options.show

    if not fileExist(input_list):
      sys.stderr.write("Missing input file %s \n"%input_list)
      exit(0)

    if not fileExist(fasta_file):
      sys.stderr.write("Missing FastA file  %s \n"%fasta_file)
      exit(0)

    #Dictionary with results
    new_models={}
    group_models={}
    #Select the active models
    if verbose: sys.stdout.write('\t\t-- Open list of models %s ...\n'%input_list)
    fd=open(input_list,"r")
    models={}
    for line in fd:
      data=line.strip().split('\t')
      if data[2]=="DONE" or data[2]=="FIXED":
       pair=(data[0],data[1])
       if models.has_key(pair):
          models.pop(pair)
          models.setdefault(pair,data[3])
       else:
          models.setdefault(pair,data[3])
          if verbose: sys.stdout.write('\t\t\t-- Add models for %s from %s ...\n'%(pair,data[3]))
    fd.close()

    #If we need to renumerate PDBs we need to read the FastA file
    proteins= {}
    if verbose: sys.stdout.write('\t\t-- Reading protein sequences to renumerate files...\n')
    for protein in FASTA_iterator(fasta_file):
            name = protein.get_identifier()
            if len(name.split("|"))>1:
               name1 = name.split("|")[1] 
               name2 = name.split("|")[2] 
            if len(name.split("|")) == 2:
               name1 = name.split("|")[1]
               name2 = name.split("|")[1]
            if len(name.split("|"))<=1:
               name1 = name
               name2 = name
            seq  = protein.get_sequence()
            if len(seq) > 0:
                try:
                    proteins.setdefault(name1, ProteinSequence(name1, seq))
                    proteins.setdefault(name2, ProteinSequence(name2, seq))
                except IncorrectSequenceLetter as e:
                    sys.stderr.write('WARNING: %s\n' %e)
                    sys.stdout.write('\t\t-- Skip input sequence: %s Sequence: %s\n' %(name, seq))


    #If we need to add hydrogens / renumerate PDBs
    if options.hbplus:
      for pair,folder in models.iteritems():
        fo=open(folder+".h","w")
        fd=open(folder,"r")
        for line in fd:
         path_array= line.strip().split("/")
         path_file = "/".join([str(x) for x in path_array[:-1]])
         input_file=path_array[-1]
         if not fileExist(os.path.join(path_file,input_file)): continue
         if input_file.split(".")[-1]=="pdb":
          output_file=path_file+"/"+".".join([str(x) for x in input_file.split(".")[:-1]])+".h"
         else:
          output_file=path_file+"/"+input_file+".h"
         if not fileExist(output_file):
          add_hydrogens(config,path_file,input_file,output_file,dummy_dir)
         fo.write("%s\n"%output_file)
         if renumerate:
          if verbose: sys.stdout.write("\t\t\t-- Renumerate residues as original sequence %s\n"%input_file)
          sequences_complex = {}
          if proteins.has_key(pair[0]) and proteins.has_key(pair[1]):
           sequences_complex.setdefault("A",proteins.get(pair[0]))
           sequences_complex.setdefault("B",proteins.get(pair[1]))
           try:
                pdb_renumber=PDB()
                pdb_renumber=renumber_pdb(config,path_file,output_file,sequences_complex,dummy_dir)
                if len(pdb_renumber.chain_identifiers)>1:
                   pdb_renumber.write(output_file,force=True)
                else:
                   if verbose: sys.stdout.write("\t\t\t-- Failed to renumerate %s\n"%output_file)
           except Exception as e:
                sys.stderr.write("WARNING %s\n"%e)
          else:
           sys.stdout.write('\t\t\t-- Skip renumbering of %s (missing FastA sequences %s %s)\n'%(output_file,pair[0],pair[1]))
        fd.close()
        fo.close()
        new_models.setdefault(pair,folder+".h")
        pose_list= os.path.join(path_file,options.label+"_"+pair[0]+"::"+pair[1]+"_cluster_"+str(0)+".list")
        shutil.copy(folder,pose_list)
        group_models.setdefault(pair,[]).append((0,pose_list))
    else:
      new_models=models
      for pair,folder in models.iteritems():
        path_array= folder.strip().split("/")
        path_file = "/".join([str(x) for x in path_array[:-1]])
        pose_list= os.path.join(path_file,options.label+"_"+pair[0]+"::"+pair[1]+"_cluster_"+str(0)+".list")
        shutil.copy(folder,pose_list)
        group_models.setdefault(pair,[]).append((0,pose_list))
        fd=open(folder,"r")
        for line in fd:
          path_array= line.strip().split("/")
          path_file = "/".join([str(x) for x in path_array[:-1]])
          input_file=path_array[-1]
          output_file=path_file+"/"+input_file
          if not fileExist(os.path.join(path_file,input_file)): continue
          if renumerate:
            if verbose: sys.stdout.write("\t\t-- Renumerate residues as original sequences (%s %s) in %s\n"%(pair[0],pair[1],input_file))
            sequences_complex = {}
            if proteins.has_key(pair[0]) and proteins.has_key(pair[1]):
             sequences_complex.setdefault("A",proteins.get(pair[0]))
             sequences_complex.setdefault("B",proteins.get(pair[1]))
             try:
                pdb_renumber=PDB()
                pdb_renumber=renumber_pdb(config,path_file,output_file,sequences_complex,dummy_dir)
                if len(pdb_renumber.chain_identifiers)>1:
                   pdb_renumber.write(output_file,force=True)
                else:
                   if verbose: sys.stdout.write("\t\t\t-- Failed to renumerate %s\n"%output_file)
             except Exception as e:
                sys.stderr.write("WARNING %s\n"%e)
            else:
             sys.stdout.write('\t\t\t-- Skip renumbering of %s (missing FastA sequences %s %s)\n'%(input_file,pair[0],pair[1]))
        fd.close()

    # Informe on the files parsed
    if options.show:
     for pair,folder in new_models.iteritems():
        sys.stdout.write("Parse: %s \n"%(folder))

    # Group files with similar interfaces
    # Defined criteria from config file
    potential=config.get('Parameters','PPI_threshold_type')
    radius   =float(config.get('Parameters','PPI_distance_threshold_shell'))
    overlap  =float(config.get('Parameters','overlap_interface'))
    #Obtain the name of the wild-type
    wt=(None,None)
    wti=(None,None)
    for a,b in new_models.iterkeys():
      wt_A = a.split("_")[0]
      wt_B = b.split("_")[0]
      wt=(wt_A,wt_B)
      wti=(wt_B,wt_A)
    if not new_models.has_key(wt) and new_models.has_key(wti): wt=wti
    #If there is no wild type select the pair with the biggest cluster to use as basic core in substitution of the wild type
    if not new_models.has_key(wt):
     maxsize=0
     for pair,folder in new_models.iteritems():
       pdbdir,interface,cluster = filter_homologs(folder,potential=potential,radius=radius,overlap=overlap,show=False)
       if max([len(s) for c,s in sorted(cluster.items(),key=lambda x: len(x[1]),reverse=True)]) >maxsize: wt=pair
    if wt==(None,None):
      return group_models 
    #Informe on the selection if "verbose"   
    if options.show:
      sys.stdout.write("\t\t -- Clustering similar interfaces for wild-type PPI %s - %s ...\n"%(wt_A,wt_B))

    #Rank the clusters of Wild Type (or select pair) by size and create the lists with grouped structures
    #Keep in the dictionary a list with the name of files (one for each pose) listing the models
    folder_wt = new_models[wt]
    pdbdir_wt,interface,cluster_wt = filter_homologs(folder_wt,potential=potential,radius=radius,overlap=overlap,show=False)
    if options.show: sys.stdout.write("\t\t\t Directory %s\n"%pdbdir_wt)

    #Write a list of non redundant interfaces for wt
    fg=open(os.path.join(pdbdir_wt,options.label+"_"+wt[0]+"::"+wt[1]+"_nr_interfaces.list"),"w")
    #Pose 0 is reserved for all structures wityhout clustering, next steps pose will increase by 1
    pose=0
    pose_wt=[]
    for c,s in sorted(cluster_wt.items(),key=lambda x: len(x[1]),reverse=True): 
        if len(s) <=0: continue
        pose     = pose + 1
        pose_list= os.path.join(pdbdir_wt,options.label+"_"+wt[0]+"::"+wt[1]+"_cluster_"+str(pose)+".list")
        if options.show: sys.stdout.write("\t\t\t Seed cluster %s of wild-type is in POSE %d\n"%(c,pose))
        pose_wt.append(c)
        group_models.setdefault(wt,[]).append((pose,pose_list))
        fd=open(pose_list,"w")
        grouped="\t"
        for p in s:
          pdb=pdbdir_wt+"/"+p
          if not fileExist(pdb): continue
          fd.write("%s\n"%(pdb))
          if p != c:
            z=0.0
            if len(interface[c]) >0:
              z=100 * float(len(interface[c].intersection(interface[p])))/len(interface[c])
            pb=p.split(".")
            grouped += ".".join([str(x) for x in pb[1:-1]])
            grouped += " ({0:5.1f})\t".format(z)
        fd.close()
        fg.write("%s\t%s\n"%(pdbdir_wt+"/"+c,grouped))
    fg.close()
    pose_last=pose

    #Compare and group similar poses to the wt for the other pairs
    #the remaining poses are ranked at the end by the size of the clusters
    min_ratio_common_poses=float(config.get('Parameters', 'min_ratio_common_poses'))
    for pair,folder in new_models.iteritems():
       if pair==wt: continue

       if options.show: sys.stdout.write("\t\t -- Clustering similar interfaces for PPI %s - %s ...\n"%(pair[0],pair[1]))
       pdbdir,interface,cluster = filter_homologs(folder,potential=potential,radius=radius,overlap=overlap,show=False)
       if options.show: sys.stdout.write("\t\t\t Directory %s\n"%pdbdir)

       #Select the poses of wt that correspond to each cluster of similar interfaces
       select={}
       for c,s in sorted(cluster.items(),key=lambda x: len(x[1]),reverse=True):
        if len(s) <=0: continue
        pose_pair=None
        maxcommon=0
        ratio_common=0
        for ipose in xrange(1,len(pose_wt)+1):
         if ipose in select.itervalues(): continue
         s_wt=cluster_wt[pose_wt[ipose-1]]
         main_wt,main_s=compare_templates(s_wt,s)
         if len(main_wt.intersection(main_s))>maxcommon: 
            pose_pair=ipose
            maxcommon=len(main_wt.intersection(main_s))
            ratio_common=float(maxcommon)/len(main_wt)
           # if options.show: sys.stdout.write("\t\t\t Ratio of common poses %f (%d/%d) with pose %d \n"%(ratio_common,maxcommon,len(main_wt),ipose))
            if maxcommon > len(main_wt):
               if options.show:sys.stdout.write("Error too large number of common poses %s compared with %s\n"%(repr(main_wt.intersection(main_s)),repr(main_wt)))
        if pose_pair is not None and ratio_common > min_ratio_common_poses:  
           select.setdefault(c,pose_pair)
           if options.show: sys.stdout.write("\t\t\t Seed cluster %s in wild-type POSE %d (ratio %f)\n"%(c,pose_pair,ratio_common))
        
       #To continue other poses after the last one use the iterator pose
       pose=pose_last

       #Write a list of non redundant interfaces for  the pair
       fg=open(os.path.join(pdbdir,options.label+"_"+pair[0]+"::"+pair[1]+"_nr_interfaces.list"),"w")

       for c,s in sorted(cluster.items(),key=lambda x: len(x[1]),reverse=True):
        if len(s) <=0: continue

        #Print the list of the selected pose and add to the list in group_models dictionary
        if select.has_key(c):
         pose_pair=select[c]
         pose_list= os.path.join(pdbdir,options.label+"_"+pair[0]+"::"+pair[1]+"_cluster_"+str(pose_pair)+".list")
         group_models.setdefault(pair,[]).append((pose_pair,pose_list))
         fd=open(pose_list,"w")
         grouped="\t"
         for p in s:
          pdb=pdbdir+"/"+p
          if not fileExist(pdb): continue
          fd.write("%s\n"%(pdb))
          if p != c:
            z=0.0
            if len(interface[c]) >0:
              z=100 * float(len(interface[c].intersection(interface[p])))/len(interface[c])
            pb=p.split(".")
            grouped += ".".join([str(x) for x in pb[1:-1]])
            grouped += " ({0:5.1f})\t".format(z)
         fd.close()
         fg.write("%s\t%s\n"%(pdbdir+"/"+c,grouped))

        #Print the list for a new pose and add to the list in group_models dictionary
        else:
         pose     = pose + 1
         pose_list= os.path.join(pdbdir,options.label+"_"+pair[0]+"::"+pair[1]+"_cluster_"+str(pose)+".list")
         group_models.setdefault(pair,[]).append((pose,pose_list))
         if options.show: sys.stdout.write("\t\t\t Seed cluster %s in new POSE %d \n"%(c,pose))
         fd=open(pose_list,"w")
         grouped="\t"
         for p in s:
          pdb=pdbdir+"/"+p
          if not fileExist(pdb): continue
          fd.write("%s\n"%(pdb))
          if p != c:
            z=0.0
            if len(interface[c]) >0:
              z=100 * float(len(interface[c].intersection(interface[p])))/len(interface[c])
            pb=p.split(".")
            grouped += ".".join([str(x) for x in pb[1:-1]])
            grouped += " ({0:5.1f})\t".format(z)
         fd.close()
         fg.write("%s\t%s\n"%(pdbdir+"/"+c,grouped)) 
       fg.close()
       #Keep the number of last pose for the analysis of next pairs
       pose_last=pose

    return group_models

def interface_affected(pair):

    x,y=pair
# x,y are the ratio of interfaces with different sequence with respect to a wild_type form interaction of proteins a and b, espectively

    ratio_a = int(10*x)
    ratio_b = int(10*y)
# version for 2 ratios
#    return "_i"+str(ratio_a)+"&"+str(ratio_b)
# version single value
    return "_i"+str(max([ratio_a,ratio_b]))

def compare_interface_sequences(list_of_models,dummy_dir):

#Result dicitionary
    pose_interfaces={}
#Define parameters to calculate the interface
    PPI_threshold_type = config.get('Parameters', 'PPI_threshold_type')
    PPI_distance_threshold = float(config.get('Parameters', 'PPI_distance_threshold'))

#Select the name of the wild-type pair of sequences for each pair
#wt_of_pair is a dictionary that assigns for each pair the corresponding wild-type
#indicating if the order of proteins is the same "+1" or inverse "-1", or if there is no wild-type to compare "0"
    wt_of_pair={}
    for pair,cluster_list in list_of_models.iteritems():
      a,b = pair
      wt_A = a.split("_")[0]
      wt_B = b.split("_")[0]
      wt=(wt_A,wt_B)
      wti=(wt_B,wt_A)
      if list_of_models.has_key(wt):
        wt_of_pair.setdefault(pair,(wt,1))
      elif list_of_models.has_key(wti):
        wt_of_pair.setdefault(pair,(wti,-1))
      else:
        wt_of_pair.setdefault(pair,(pair,0))

#Get the list of pair of reference-sequences of each pose and their wild-type references

    main_sequences,main_sequences_wt = extract_main_sequence(list_of_models,wt_of_pair)

# Chek sequences of different forms with respect to wild-type fo each pose
# defined names of use
# a,b is the name of the pair of interacting proteins
# pose is the cluster number to be checked
# seq_a and seq_b are the main sequences of a and b for the corresponding pose
# pa and pb are the ratio of occurrence of the main sequences used for a and b in the  corresponding pose
# wt_a,seq_wt_a,pwa and  wt_b,seq_wt_b,pwb are the names sequecnes and ratio of occurrence of the wild-type
#   protein names corresponding in the pose for a and b (this can change in diferent poses, not the name but the main sequence)
# check_A is the set of 2-tuples aa,residue_number different of sequence A in pose respect to the main wild-type sequence
# check_wt_A is the set of 2-tuples aa,residue_number different of wild type sequence of A respect to A in pose
# idem for check_B and check_wt_B
# Information is duplicated, with check_wt_A and check_wt_B is enough
#
    check_sequence={}
    for ppairs,spairs in main_sequences.iteritems():
        pose,pair      = ppairs
        a,b            = pair
        sp_a,sp_b      = spairs
        seq_a,pa       = sp_a
        seq_b,pb       = sp_b
        wt,order       = wt_of_pair[(a,b)]
        if not main_sequences_wt.has_key((pose,wt)): continue
        s_wt_a,s_wt_b  = main_sequences_wt[(pose,wt)]
        if order > 0:
          wt_a,wt_b    = wt
          seq_wt_a,pwa = s_wt_a
          seq_wt_b,pwb = s_wt_b
        if order < 0:
          wt_b,wt_a    = wt
          seq_wt_a,pwa = s_wt_b
          seq_wt_b,pwb = s_wt_a
        try:
         check_a, check_wt_A = compare_seq_wt(a,seq_a,wt_a,seq_wt_a,dummy_dir)
         check_b, check_wt_B = compare_seq_wt(b,seq_b,wt_b,seq_wt_b,dummy_dir)
         check_sequence.setdefault(ppairs,(check_a, check_wt_A, check_b, check_wt_B))
        except:
         sys.stderr.write("WARNING: Skip pair %s %s\n"%(a,b))

# Get interface aminoacids of Wild-type for each pose
# interface is a dictionary with the pose and the pair as key, 
# the values are 3-tuples: two Counter comprising the data of the lists of Aa in the interface and the number of PDB files in the cluster
    interface_wt={}
    for ppairs,spairs in main_sequences_wt.iteritems():
        pose,pair      = ppairs
        a,b            = pair
        sp_a,sp_b      = spairs
        seq_a,pa       = sp_a
        seq_b,pb       = sp_b
        cluster_list_wt= list_of_models[pair]
        for pose_wt,pose_list_wt in cluster_list_wt:
          if pose!=pose_wt: continue
          if interface_wt.has_key((pair,pose_wt)): continue
          filelist=open(pose_list_wt,"r")
          interface_a=[]
          interface_b=[]
          n=0
          for line in  filelist:
            n=n+1
            pdb_file=line.strip()
            if not fileExist(pdb_file):continue
            pdb=PDB(pdb_file)
            chains=[x for x in pdb.chain_identifiers]
            a_index={}
            b_index={}
            p=pdb.get_chain_by_id(chains[0])
            for k,idx in enumerate(p.protein_idx.split(";")): a_index.setdefault(idx.replace(" ",""),k)
            q=pdb.get_chain_by_id(chains[1])
            for k,idx in enumerate(q.protein_idx.split(";")): b_index.setdefault(idx.replace(" ",""),k)
            #check these are main-sequences for analysis
            if p.protein_sequence != seq_a or q.protein_sequence != seq_b : continue
            # Check contacts
            protein_complex = Complex(pdb, PPI_type = PPI_threshold_type, PPI_distance = PPI_distance_threshold)
            for interaction in  protein_complex.PPInterfaces:
                 order_complex=0
                 if interaction.interactor_id.split("_")[-1] == chains[1] and interaction.protein_id.split("_")[-1] == chains[0]: order_complex=1
                 if interaction.interactor_id.split("_")[-1] == chains[0] and interaction.protein_id.split("_")[-1] == chains[1]: order_complex=-1
                 if order_complex==0: continue
                 if order_complex>0:
                    for aa in interaction.interactor_positions: interface_b.append((aa.single_letter, b_index[aa.identifier.replace(" ","")]))
                    for aa in interaction.protein_positions: interface_a.append((aa.single_letter, a_index[aa.identifier.replace(" ","")]))
                 if order_complex<0:
                    for aa in interaction.interactor_positions: interface_a.append((aa.single_letter, a_index[aa.identifier.replace(" ","")]))
                    for aa in interaction.protein_positions: interface_b.append((aa.single_letter, b_index[aa.identifier.replace(" ","")]))
          #After checking all the structures in the list
          filelist.close()
          #Add the counter dictionary of the list of pairs Aa,residue_number of the interfaces between a and b chains 
          interface_wt.setdefault(ppairs,(Counter(interface_a),Counter(interface_b),n))

#Check if residues involved in the differences with respect to wild-type are also involved in the interface of the wt                   
    
    for pair,cluster_list in list_of_models.iteritems():
      a,b = pair
      wt,order = wt_of_pair[(a,b)]
      #check only pairs with known wild-type
      if order == 0: continue  
      for pose,pose_list in cluster_list:
         ppair     =(pose,pair)
         wt_pair   =(pose,wt)
         #check exist wild-type interface for this pose
         if not interface_wt.has_key(wt_pair): continue
         if not main_sequences.has_key(ppair): continue
         sp_a,sp_b = main_sequences[ppair]
         seq_a,ra  = sp_a
         seq_b,rb  = sp_b
         check_a, check_wt_A, check_b, check_wt_B         = check_sequence[ppair]
         interface_wt_A, interface_wt_B, number_wt_models = interface_wt[wt_pair]
         ratio_A=0
         ratio_B=0
         if number_wt_models > 0:
           affected_residue_A=check_wt_A.intersection(interface_wt_A.keys())
           if len(affected_residue_A) >0:
             largest_number_A = max([interface_wt_A[x] for x in affected_residue_A])
             ratio_A = float(largest_number_A) / number_wt_models
           affected_residue_B=check_wt_B.intersection(interface_wt_B.keys())
           if len(affected_residue_B)>0:
             largest_number_B = max([interface_wt_B[x] for x in affected_residue_B])
             ratio_B = float(largest_number_B) / number_wt_models
           pose_interfaces.setdefault(ppair,(ratio_A,ratio_B))
#return result
    return pose_interfaces

def compare_seq_wt(name,seq,name_wt,seq_wt,dummy_dir):
    clustal_exe  = os.path.join(config.get('Paths','clustal_path'),'clustalw2')
    #create files

    infile=dummy_dir+"/tmp_"+name_wt+"_"+name+".fa"
    outfile=dummy_dir+"/tmp_"+name_wt+"_"+name+".aln"
    dndfile=dummy_dir+"/tmp_"+name_wt+"_"+name+".dnd"
    fd=open(infile,"w")
    fd.write(">WT{0:s}\n{1:s}\n".format(name_wt,seq_wt))
    fd.write(">{0:s}\n{1:s}\n".format(name,seq))
    fd.close()

    try:
      # run clustalw2
      msa_cline=ClustalwCommandline(clustal_exe,infile=infile,outfile=outfile)
      child = subprocess.Popen(str(msa_cline),stdout=subprocess.PIPE,stderr=subprocess.PIPE,shell="/bin/bash")
      child.communicate()
      #store alignment in compare
      compare=AlignIO.read(outfile,'clustal')
    except Exception as e:
      sys.stderr.write("ERROR: %s\n"%e)
      return (set(),set())

    #remove temporary fasta and alignment files
    remove_files([infile,outfile,dndfile])

    wt_msa =compare[0].seq
    seq_msa=compare[1].seq
    try:
      len_wt=len(seq_wt)
      len_msa=len(wt_msa)
    except  Exception as e:
      sys.stderr.write("ERROR and SKIP: %s\n"%e)
      return (set(),set())
    sys.stdout.write("\t\t -- Sequence comparison of Wild-Type %s versus the form %s\n"%(name_wt,name))
    difference=""
    for ii in itertools.izip(wt_msa,seq_msa):
      if ii[0]==ii[1]:difference=difference+" "
      else:           difference=difference+"*"
    sys.stdout.write("\t\t\t%s\n\t\t\t%s\n\t\t\t%s\n"%(wt_msa,seq_msa,difference))
    #Check sequence affected of wild-type
    residue_number_wt=0
    k=0
    affected_wt_residue=set()
    while (residue_number_wt<len(seq_wt)) and (k<len(wt_msa)):
      if wt_msa[k] == seq_wt[residue_number_wt] and seq_msa[k]==wt_msa[k]:
         residue_number_wt = residue_number_wt + 1
         k = k + 1
      elif  wt_msa[k] == "-":
         affected_wt_residue.add((seq_wt[residue_number_wt],residue_number_wt))
         if residue_number_wt>0: affected_wt_residue.add((seq_wt[residue_number_wt-1],residue_number_wt-1))
         k = k + 1
      elif  seq_msa[k] !=  wt_msa[k] and wt_msa[k] == seq_wt[residue_number_wt]:
         affected_wt_residue.add((seq_wt[residue_number_wt],residue_number_wt))
         residue_number_wt = residue_number_wt + 1
         k = k + 1
      else:
         print("Error: no matching sequences %s [%d] vs %s [%d]\n"%(wt_msa[k],k,seq_wt[residue_number_wt],residue_number_wt))
         residue_number_wt = residue_number_wt + 1
         k = k + 1

    #Check sequence affected of sequence under test
    residue_number=0
    k=0
    affected_residue=set()
    try:
      len_seq=len(seq)
      len_msa=len(seq_msa)
    except  Exception as e:
      sys.stderr.write("ERROR and SKIP: %s\n"%e)
      return (set(),set())

    while (residue_number<len(seq)) and (k<len(seq_msa)):
      if seq_msa[k] == seq[residue_number] and seq_msa[k]==wt_msa[k]:
         residue_number = residue_number + 1
         k = k + 1
      elif  seq_msa[k] == "-":
         affected_residue.add((seq[residue_number],residue_number))
         if residue_number>0: affected_residue.add((seq[residue_number-1],residue_number-1))
         k = k + 1
      elif  seq_msa[k] != wt_msa[k] and  seq_msa[k] == seq[residue_number]:
         affected_residue.add((seq[residue_number],residue_number))
         residue_number = residue_number + 1
         k = k + 1
      else:
         print("Error: no matching sequences %s [%d] vs %s [%d]\n"%(seq_msa[k],k,seq[residue_number],residue_number))
         residue_number_wt = residue_number_wt + 1
         k = k + 1
        
    return (affected_residue,affected_wt_residue)
     



def extract_main_sequence(list_of_models,wt_of_pair):

    main_sequences={}
    main_sequences_wt={}
    for pair,cluster_list in list_of_models.iteritems():
        a,b = pair
        for pose,pose_list in cluster_list:
            sequence={}
            seq_a=None
            seq_b=None
            ratio_a=0.0
            ratio_b=0.0
            filelist=open(pose_list,"r")
            for line in  filelist:
              pdb_file=line.strip()
              if not fileExist(pdb_file):continue
              try:
                pdb=PDB(pdb_file)
                chains=[x for x in pdb.chain_identifiers]
                p=pdb.get_chain_by_id(chains[0])
                q=pdb.get_chain_by_id(chains[1])
                sequence.setdefault(a,[]).append(p.protein_sequence)
                sequence.setdefault(b,[]).append(q.protein_sequence)
              except Exception as e:
                sys.stdout.write("WARNING: file not found, skip %s \n"%pdb_file)
                sys.stderr.write("ERROR: %s\n"%e)
            filelist.close()
            if sequence.has_key(a): 
              seq_a,times_a=Counter(sequence[a]).most_common(1)[0]
              ratio_a=float(times_a)/len(sequence[a])
            if sequence.has_key(b): 
              seq_b,times_b=Counter(sequence[b]).most_common(1)[0]
              ratio_b=float(times_b)/len(sequence[b])
            main_sequences.setdefault((pose,pair),((seq_a,ratio_a),(seq_b,ratio_b)))
            wt,order= wt_of_pair[pair]
            if pair == wt:
                  main_sequences_wt.setdefault((pose,wt),((seq_a,ratio_a),(seq_b,ratio_b)))
    main_sequences_all=( main_sequences , main_sequences_wt)
    return main_sequences_all

def parse_user_arguments(*args, **kwds):

    parser = argparse.ArgumentParser(
        description = 'Energy analysis of protein-protein interactions modelled with ModPPI',
        epilog      = '@Oliva\'s lab 2016')
    parser.add_argument('-ppi', '--ppi_list', dest = 'ppi_list', action = 'store',default=None,
                        help = 'File with models done (i.e. "interactions_done.list" )')
    parser.add_argument('-seq', '--fasta', dest = 'fasta_file', action = 'store',
                        help = 'Input file with a list of FASTA sequences of the proteins to test')
    parser.add_argument('-l', '--label_name', dest = 'label', action = 'store', default = "ModPPI",
                        help = 'Label to store the analyses (default is "ModPPI")')
    parser.add_argument('-o', '--output_dir', dest = 'outdir', action = 'store', default = None,
                        help = 'Directory to store the results (default is for global outputs "analysis" and for each model the same as the directory with the 3D-models')
    parser.add_argument('-d', '--dummy_dir', dest = 'dummy_dir', action = 'store', default = '/tmp/modppi_dummy',
                        help = 'Specifies the location of the dummy folder (default is /tmp/modppi_dummy)')
    parser.add_argument('-zrank', '--zrank', dest = 'zrank', action = 'store_true',
                        help = 'Flag to calculate energies with ZRANK (default is False)')
    parser.add_argument('-foldx', '--foldx', dest = 'foldx', action = 'store_true',
                        help = 'Flag to calculate energies with FoldX (default is False)')
    parser.add_argument('-rosetta', '--rosetta', dest = 'rosetta', action = 'store_true',
                        help = 'Flag to calculate energies with ROSETTA (default is False)')
    parser.add_argument('-ssp', '--split-potentials', dest = 'ssp', action = 'store_true',
                        help = 'Flag to calculate energies with Split-Statistic Potentials (default is False unless no other method is used)')
    parser.add_argument('-hydro','--hydrogens', dest = 'hbplus', action = 'store_true',
                        help = 'Flag to include hydrogens (default is False)')
    parser.add_argument('-boxplot','--boxplot', dest = 'boxplot', action = 'store_true',
                        help = 'Flag to make default boxplots (default is False)')
    parser.add_argument('-r','--renumerate', dest = 'renumerate' , action = 'store_true',
                        help = 'Flag to renumber the sequences as in the original FastA (default is False)')
    parser.add_argument('-v', '--verbose', dest = 'show', action = 'store_true',
                        help = 'Flag for verbose mode (default is False)')
    options = parser.parse_args()
    return options


if __name__ == '__main__':
    main()





from datetime import date

tables = {'sources':'source_databases'}

class Source(object):
    def __init__(self, name, source):
        self.name   = name
        self.source = source
        self.date   = date.today()

    @property
    def date2string(self):
        return '{0}_{1}_{2}'.format(self.date.strftime('%d'), self.date.strftime('%b'), self.date.strftime('%y'))

    @property
    def date2SQL(self):
        return '{0}-{1}-{2}'.format(self.date.strftime('%Y'), self.date.strftime('%m'), self.date.strftime('%d'))

    def toSQL(self):
        command  = ""
        command += "INSERT INTO {0} VALUES ('{1.name}','{1.source}','{1.date2SQL}')\n".format(tables['sources'], self)
        command += "ON DUPLICATE KEY UPDATE "
        command += "{0}.date='{1.date}';\n".format(tables['sources'], self)
        return command


'''
@file: file.py

@author: Jaume Bonet
@mail:   jaume.bonet@gmail.com
@date:   02/2013

@ [oliva's lab](http://sbi.imim.es)

@class: File
@class: FileError
'''

import os
import math
import gzip
import zipfile
import tarfile
import bz2

from . import Butler
SBIg = Butler()


class File(object):
    '''
    Manages certain aspects of the IO process.

    It allows to seamless control some aspects about working with files.
    Namely, it will allow to:
        * work with regular, tar, zip, gzip and bzip2 files.
        * quickly obtain paths, file name, extension...
        * check the viability reading/writing a given file
        * avoid/ignore overwrite an existing file
        * split a file into several
        * get files contained inside (for tar and zip) and extract them

    '''
    EMPTY_ACTION     = frozenset(['n'])
    WRITE_ACTION     = frozenset(['w', 'a', 'ar', 'wb'])
    READ_ACTION      = frozenset(['r', 'rb', 'r|*', 'r|'])
    AVAILABLE_ACTION = WRITE_ACTION.union(READ_ACTION, EMPTY_ACTION)

    TAR_EXTENSIONS    = frozenset(['.tar',    '.tgz', '.tb2', '.tbz2',
                                   '.tar.gz', '.tar.bz2'])
    GZIP_EXTENSIONS   = frozenset(['.gz',  '.tar.gz', '.tgz'])
    BZIP_EXTENSIONS   = frozenset(['.bz2', 'tar.bz2', '.tb2', '.tbz2'])
    ZIP_EXTENSIONS    = frozenset(['.zip'])

    ACTIVE_EXTENSIONS = TAR_EXTENSIONS.union(GZIP_EXTENSIONS,
                                             BZIP_EXTENSIONS,
                                             ZIP_EXTENSIONS)

    def __init__(self, file_name, action = 'r', overwrite = None):
        '''
        @param:    file_name
        @pdef:     File name.
        @ptype:    {String}

        @param:    action
        @pdef:     Whether we are opening a file to read ('r') or write ('w').
        @pdefault: 'r'
        @ptype:    {String}

        @param:    overwrite
        @pdef:     For writing actions. Decides whether it can overwrite an
                   existing file.
        @pdefault: _SBIglobals.overwrite_
        @ptype:    {Boolean}

        @raises: {FileError} after several checks.
        '''
        self._file  = file_name
        self._error = FileError(file_name)

        SBIg.alert(0, 'Opening: {0}'.format(self.full))

        self._type  = ''
        self._multi = 1
        self._check_file_type()

        SBIg.alert(2, '\tFile is {0}'.format(self._type))
        SBIg.alert(2, '\tContains {0} file'.format(self._multi))

        # The selected action must be valid according to the active extension
        # or simply to a regular file.
        self._action = None
        self._check_action(action.lower())

        SBIg.alert(1, '\tApply Action {0}'.format(self._action))

        # Local overwrite takes precedence over Global overwrite
        self._overwrite = SBIg.decide_overwrite(overwrite)

        ostring = 'overwrite' if self._overwrite else 'not overwrite'
        if not self._action.startswith('r'):
            SBIg.alert(1, '\tMode set to {0}'.format(ostring))

        # Check that the requested action can be performed over that
        # particular file
        self._check_file()

        self._is_open = False
        self._fd      = None

        # For files generated via split
        self._section = None

        self.open()

    ##############
    # ATTRIBUTES #
    ##############
    @property
    def full(self):
        '''
        Full file path.

        @return:  {String}
        '''
        return os.path.abspath(self._file)

    @property
    def relative(self):
        '''
        Path relative to the current working directory

        @return: {String}
        '''
        return os.path.relpath(self.full)

    @property
    def dir(self):
        '''
        Full file directory path.

        @return:  {String}
        '''
        return os.path.split(self.full)[0]

    @property
    def lastdir(self):
        '''
        Last directory name.

        @return:  {String}
        '''
        return os.path.basename(self.dir)

    @property
    def name(self):
        '''
        File name. Without path.

        @return:  {String}
        '''
        return os.path.basename(self.full)

    @property
    def prefix(self):
        '''
        File name. Without extension.

        @return:  {String}
        '''
        return os.path.splitext(self.name)[0]

    @property
    def first_prefix(self):
        '''
        File name. First section before a '.'.

        @return:  {String}
        '''
        return self.name.split('.')[0]

    @property
    def extension(self):
        '''
        File extension.

        @return:  {String}
        '''
        return os.path.splitext(self.name)[-1]

    @property
    def action(self):
        '''
        File selected action

        @return:  {String}
        '''
        return self._action

    @property
    def descriptor(self):
        msg = 'Use read() instead of descriptor'
        SBIg.warn(msg)
        return self._fd

    @property
    def size(self):
        '''
        File size.

        @return:  {String}
        '''
        return os.path.getsize(self.full)

    @property
    def split_section(self):
        '''
        Value when the file has been generated by spliting another file_name

        @return: {String}
        '''
        return self._section

    @split_section.setter
    def split_section(self, value):
        '''
        @param:    value
        @pdef:     new value for the attribute
        @ptype:    {String}
        '''
        self._section = value

    ############
    # BOOLEANS #
    ############
    @property
    def is_regular_file(self):
        '''
        @return:  {Boolean}
        '''
        return self._type == 'regular'

    @property
    def is_tarfile(self):
        '''
        @return:  {Boolean}
        '''
        return self._type.startswith('tar')

    @property
    def is_compressed(self):
        '''
        @return:  {Boolean}
        '''
        return self.is_gzipped or self.is_zipped or self.is_bzipped

    @property
    def is_gzipped(self):
        '''
        @return:  {Boolean}
        '''
        return self._type.endswith('gzip')

    @property
    def is_zipped(self):
        '''
        @return:  {Boolean}
        '''
        return self._type.endswith('zip') and not self._type.endswith('gzip')

    @property
    def is_bzipped(self):
        '''
        @return:  {Boolean}
        '''
        return self._type.endswith('bzip2')

    @property
    def has_multiple_files(self):
        '''
        @return:  {Boolean}
        '''
        return self._multi > 1

    ###########
    # METHODS #
    ###########
    def open(self):
        '''
        Upon starting a {File} object, it needs to be opened.

        @return: {file descriptor}
        '''
        if self._is_open:
            return

        self._is_open = True
        if self.is_regular_file:
            self._fd = open(self.full, self.action)
        if self.is_gzipped and not self.is_tarfile:
            self._fd = gzip.open(self.full, self.action)
        if self.is_bzipped and not self.is_tarfile:
            self._fd = bz2.BZ2File(self.full, self.action)
        if self.is_tarfile:  # Only to read
            self._fd = tarfile.open(self.full)
        if self.is_zipped:   # Only to read
            self._fd = zipfile.ZipFile(self.full)

    def close(self, clean = False):
        '''
        Closes the file once we are done with it

        @param:    clean
        @pdef:     erase the file on closing if file size is 0.
        @pdefault: _False_
        @ptype:    {Boolean}

        @warns: that files will be deleted if _clean_ is _True_
        '''
        self._fd.close()
        if clean and self.size == 0:   # Delete empty files
            w = '{0} is empty and it\'s going to be deleted'.format(self.full)
            SBIg.warn(w)
            os.unlink(self.full)
        self._is_open = False

    def read(self):
        '''
        Read from file.

        @raises: {FileError} if wrong action or multiple files
        @return: {file descriptor}
        '''
        if not self._action in File.READ_ACTION:
            raise self._error.wrong_action('read')
        if self.has_multiple_files:
            raise self._error.multiple()

        if self.is_regular_file or self.is_gzipped:
            return self._fd
        if self.is_bzipped:
            return self._fd.read()
        if self.is_zipped:
            return self._fd.read(self._fd.namelist()[0]).split('\n')
        if self.is_tarfile:
            return self._fd.extractfile(self._fd.getmembers()[0])

    def skip_lines(self, number):
        '''
        Activate to skip n lines before reading

        @param:    number
        @pdef:     lines to skip
        @ptype:    {Integer}
        '''
        for i in range(number):
            self._fd.readline()

    def read_file(self, file_name):
        '''
        Reads a specific file inside a multi-file container.

        @param: file_name
        @pdef:  name of the internal file to read.
        @ptype: {String}

        @raises: {FileError} if file_name not in container or
                 if not multi-file.
        @return: {file descriptor}
        '''
        if not self.has_multiple_files:
            raise self._error.no_multiple()
        if self.is_zipped:
            if file_name not in self._fd.namelist():
                raise self._error.no_file_in_file(file_name)
            return self._fd.read(file_name).split('\n')
        if self.is_tarfile:
            if file_name not in self._fd.getnames():
                raise self._error.no_file_in_file(file_name)
            return self._fd.extractfile(self._fd.getmember(file_name))

    def has_file(self, file_name):
        '''
        Checks if a specific file exists inside a multi-file container.

        @param: file_name
        @pdef:  name of the internal file to read.
        @ptype: {String}

        @raises: {FileError} if not multi-file.
        @return: {Boolean}
        '''
        if not self.has_multiple_files:
            raise self._error.no_multiple()
        if self.is_zipped:
            return file_name in self._fd.namelist()
        if self.is_tarfile:
            return file_name in self._fd.getnames()

    def write(self, line):
        '''
        Write into the file.

        @param: line
        @pdef:  content to write in the file.
        @ptype: {String}

        @raises: {FileError} if wrong action
        '''
        if not self._action in File.WRITE_ACTION:
            raise self._error.wrong_action('write')
        self._fd.write(line)

    def flush(self):
        '''
        Force to flush the contained data

        @raises: {FileError} if wrong action
        '''
        if not self._action in File.WRITE_ACTION:
            raise self._error.wrong_action('write')
        self._fd.flush()

    def list_files(self):
        '''
        List internal files inside a compressed or tar file.

        @raises: {FileError} if file does not contain multiple files.
        @return: {List} of file names
        '''
        if not self.has_multiple_files:
            raise self._error.no_multiple()
        else:
            if self.is_tarfile:
                return self._fd.getnames()
            if self.is_zipped:
                return self._fd.namelist()

    def split(self, size    = 1073741824, ini_separator        = None,
              end_separator = None, sections = None, overwrite = None,
              target_dir    = os.getcwd(), compress            = False):
        '''
        Splits a given file in multiple files.
        Splitting can be performed by file size, number of expected files
        and/or according to a given separator. Number of expected files takes
        precedence over size.

        A defined separator will avoid splitting data.
        For example, uniprot files separate between proteins with '//',
        then by declaring and end separator we can avoid a protein info from
        being split between two files.
        Similarly, fasta info starts with '>'. Thus, declaring a initial
        separator will ensure not to split a protein sequence.

        @param:    size
        @pdef:     Max size expected for a file. IN BYTES.
        @pdefault: 1073741824. (1GB).
        @ptype:    {Integer}

        @param:    ini_separator
        @pdef:     Separator definition.
        @pdefault: _None_
        @pclash:   end_separator
        @ptype:    {String}

        @param:    end_separator
        @pdef:     Separator definition.
        @pdefault: _None_
        @pclash:   ini_separator
        @ptype:    {String}

        @param:    sections
        @pdef:     Number of expected files.
        @pdefault: _None_
        @ptype:    {Integer}

        @param:    overwrite
        @pdef:     Overwrite files if they already exist.
        @pdefault: _SBIglobals.overwrite_
        @ptype:    {Boolean}

        @param:    target_dir
        @pdef:     Directory to which the new files will be created.
        @pdefault: Current working directory.
        @ptype:    {String}

        @param:    compress
        @pdef:     New files will be gzipped.
        @pdefault: _False_
        @ptype:    {Boolean}

        @warns:  If self is a multi-file container, it will ignore parameters
                 and call self.extract.
        @raises: {FileError} if can not write in target directory
        @raises: {AttributeError} if both start_separator and end_separator
                 are defined OR if size AND sections are both not defined,
                 OR if size OR sections are not Integers.
        '''
        if self.has_multiple_files:
            SBIg.warn('For multi-file containers use File.extract()')
            self.extract(target_dir)
        if not os.path.isdir(target_dir) or not os.access(target_dir, os.W_OK):
            raise self._error.file_permissions('write', target_dir)
        if (size is None and sections is None):
            errmsg = 'Either size or sections must be defined.\n'
            raise AttributeError(errmsg)
        if (ini_separator is not None and end_separator is not None):
            errmsg = 'Both separators can not be defined simultaneously\n'
            raise AttributeError(errmsg)
        if size is not None:
            try:
                size = int(size)
            except:
                errmsg = 'Attribute size must be numeric.\n'
                raise AttributeError(errmsg)
        if sections is not None:
            try:
                sections = int(sections)
            except:
                errmsg = 'Attribute sections must be numeric.\n'
                raise AttributeError(errmsg)

        counter  = 1
        newfiles = []

        # section TAKES PRECEDENCE over size...
        if sections == 1:
            return [self, ]
        if sections is not None:
            size     = math.ceil(self.size / float(sections))
        else:
            sections = math.ceil(self.size / float(size))

        # preparing partition file names
        sectionid      = '{0:003d}'
        globalfilename = self.prefix + '.' + sectionid + self.extension
        outputfile     = os.path.join(target_dir, globalfilename)
        if compress:
            outputfile += '.gz'

        # Local overwrite takes precedence over Global overwrite
        overwrite = SBIg.decide_overwrite(overwrite)
        if not self._is_open:
            self.open()

        msg = 'Dividing {0.full} into {1} files (approx.)'.format(self, sections)
        SBIg.alert(0, msg)

        newfiles.append(File(file_name = outputfile.format(counter),
                             action    = 'w', overwrite = overwrite))
        newfiles[-1].split_section = sectionid.format(counter)
        for line in self.read():
            if ini_separator is not None and line.startswith(ini_separator):
                if newfiles[-1].size >= size:
                    newfiles[-1].close()
                    counter += 1
                    newfiles.append(File(file_name = outputfile.format(counter),
                                         action    = 'w', overwrite = overwrite))
                    newfiles[-1].split_section = sectionid.format(counter)

            newfiles[-1].write(line)

            if end_separator is not None and line.startswith(end_separator):
                if newfiles[-1].size >= size:
                    newfiles[-1].close()
                    counter += 1
                    newfiles.append(File(file_name = outputfile.format(counter),
                                         action    = 'w', overwrite = overwrite))
                    newfiles[-1].split_section = sectionid.format(counter)

            if end_separator is None and ini_separator is None:
                if newfiles[-1].size >= size:
                    newfiles[-1].close()
                    counter += 1
                    newfiles.append(File(file_name = outputfile.format(counter),
                                         action    = 'w', overwrite = overwrite))
                    newfiles[-1].split_section = sectionid.format(counter)

        newfiles[-1].close()
        self.close()
        self.open()
        return newfiles

    def extract(self, target_dir = os.getcwd()):
        '''
        Extract the files inside a multi-file container

        @param:    target_dir
        @pdef:     Directory to which the new files will be extracted.
        @pdefault: Current working directory.
        @ptype:    {String}

        @raises: {FileError} if can not write in target directory or if not
                 multi-file container.
        '''
        if not self.has_multiple_files:
            raise self._error.no_multiple()
        if not os.path.isdir(target_dir) or not os.access(target_dir, os.W_OK):
            raise self._error.file_permissions('write', target_dir)

        if self.is_tarfile:
            self._fd.extractall(path = target_dir)
        if self.is_zipped:
            self._fd.extractall(path = target_dir)

    ###################
    # PRIVATE METHODS #
    ###################
    def _check_file_type(self):
        '''
        Decide which kind of file we are working with.
        It uses the ACTIVE_EXTENSIONS to decide if some checking is necessary.
        '''
        if self._file.endswith(tuple(File.ACTIVE_EXTENSIONS)):
            if self._file.endswith(tuple(File.ZIP_EXTENSIONS)):
                self._type = 'zip'
                self._check_number_of_files()
            if self._file.endswith(tuple(File.TAR_EXTENSIONS)):
                self._type = 'tar'
                self._check_number_of_files()
            if self._file.endswith(tuple(File.GZIP_EXTENSIONS)):
                self._type = 'gzip' if self._type == '' else self._type + '.gzip'
            if self._file.endswith(tuple(File.BZIP_EXTENSIONS)):
                self._type = 'bzip2' if self._type == '' else self._type + '.bzip2'
        else:
            self._type = 'regular'

    def _check_number_of_files(self):
        '''
        In tar and zip files.
        Check the number of files contained inside.
        '''
        if self.is_tarfile:
            tar         = tarfile.open(self.full)
            self._multi = len(tar.getmembers())
        if self.is_zipped:
            zzip        = zipfile.ZipFile(self.full)
            self._multi = len(zzip.namelist())

    def _check_action(self, action):
        '''
        Sees that the requested action for the given file is valid and adapts
        it in case the file is compressed.

        @param: action
        @pdef:  type of action intended to apply to the File.
        @ptype: {String}

        @raises: {FileError}
        '''
        if not action in File.AVAILABLE_ACTION:  # Raise 'Wrong action' Error
            raise self._error.wrong_file_action(action)

        if self.is_tarfile and action.startswith('r'):
            action = 'r|*'

        if self.is_gzipped:
            if action.startswith('r'):
                action = 'rb'
            elif action.startswith('w'):
                action = 'wb'

        self._action = action

    def _check_file(self):
        '''
        Sees that the requested validated action can be performed over the
        selected file.

        @raises: {FileError}
        '''
        if self._action.startswith('r'):
            if os.path.isdir(self.full):
                raise self._error.is_dir()
            if not os.path.isfile(self.full):
                raise self._error.file_does_not_exist()
            if not os.access(self.full, os.R_OK):
                raise self._error.file_permissions('read')
        if self._action.startswith('w') or self._action.startswith('a'):
            if self.is_tarfile or self.is_zipped:
                raise self._error.unwritable_format()
            if os.path.isfile(self.full)and not self._overwrite:
                raise self._error.file_overwrite()
            if not os.path.isdir(self.dir):
                raise self._error.dir_does_not_exist()
            if not os.access(self.dir, os.W_OK):
                raise self._error.file_permissions('write')

    #################
    # CLASS METHODS #
    #################
    def __repr__(self):
        header = '[ {0.__class__.__name__} ][{1}]: '
        data   = '{0}{1.full}\n'.format(header.format(self,   'NAME  '), self)
        data  += '{0}{1._type}\n'.format(header.format(self,  'TYPE  '), self)
        data  += '{0}{1._multi}\n'.format(header.format(self, 'FILES '), self)
        data  += '{0}{1.action}\n'.format(header.format(self, 'ACTION'), self)
        return data

    def __str__(self):
        return self.full


class FileError(Exception):
    '''
    Manages different error produced by the {File} object when trying to
    read or write.

    '''
    def __init__(self, file_name):
        '''
        @param: file_name
        @pdef:  name of the file to which the error is related.
        @ptype: {String}
        '''
        self._file = file_name
        self._msg  = ''

    ###########
    # METHODS #
    ###########
    def wrong_file_action(self, action):
        msg  = '{0} is not an available action for file {1}.\n'
        msg += 'Accepted actions are {0}'.format(repr(File.AVAILABLE_ACTION))

        self._msg = msg.format(action, self._file)
        return self

    def file_does_not_exist(self):
        self._msg = 'File {0} does not exist.\n'.format(self._file)
        return self

    def file_permissions(self, what, where = None):
        where     = self._file if where is None else where
        self._msg = 'Permission denied to {0} {1}'.format(what, where)
        return self

    def file_overwrite(self):
        self._msg = 'Can not overwrite {0}'.format(self._file)
        return self

    def dir_does_not_exist(self):
        self._msg = 'Directory for {0} does not exist'.format(self._file)
        return self

    def is_dir(self):
        self._msg = 'Requested file {0} is a directory'.format(self._file)
        return self

    def is_file(self):
        self._msg = 'Requested directory {0} is a file'.format(self._file)
        return self

    def unwritable_format(self):
        self._msg = 'This format does not support write'.format(self._file)
        return self

    def wrong_action(self, what):
        msg  = 'Unable to {0}.\n'.format(what)
        msg += 'File {1} not in {0} mode'.format(what, self._file)

        self._msg = msg
        return self

    def no_multiple(self):
        self._msg = '{0} does not contain multiple files'.format(self._file)
        return self

    def multiple(self):
        self._msg = '{0} contain multiple files'.format(self._file)
        return self

    def no_file_in_file(self, search):
        self._msg = 'No {0} in {1}'.format(search, self._file)
        return self

    #################
    # CLASS METHODS #
    #################
    def __str__(self):
        return self._msg



'''
@file: ExternalExe.py

@author: Jaume Bonet
@mail:   jaume.bonet@gmail.com
@date:   2014

@ [oliva's lab](http://sbi.imim.es)

@class: ExternalExe
'''
import os
import ConfigParser
from os.path import join     as j
from os.path import normpath as n
from os.path import dirname  as d
from abc     import ABCMeta

from SBI.beans import Executable


class ExternalExe(object):
    '''
    Main class to derive others that will control the execution of external
    programs.

    Program configuration is defined in the configSBI.txt file or can be
    defined in a file linked to a environment variable called SBI_CONFIG_FILE.

    '''
    __metaclass__ = ABCMeta

    DEFAULT_CONFIG_FILE = j(n(d(__file__)), 'configSBI.txt')

    # Executable configuration
    _CONFIG     = ConfigParser.RawConfigParser(allow_no_value=True)
    _EXE        = None

    def __new__(cls, *args, **kwargs):
        cls._CONFIG.read(os.getenv('SBI_CONFIG_FILE', cls.DEFAULT_CONFIG_FILE))
        return super(ExternalExe, cls).__new__(cls, *args, **kwargs)

    ###################
    # PRIVATE METHODS #
    ###################
    def _set_default_executable(self, external_id):
        '''
        Configure the {Executable} of the class according to the default
        configuration as defined in the default configSBI.txt or the
        SBI_CONFIG_FILE

        @param:    external_id
        @pdef:     name of the external program as referred in the
                   configSBI.txt file.
        @ptype:    {String}
        '''
        i, e, p   = external_id, 'executable', 'path'
        self._EXE = Executable(executable= self._CONFIG.get(i, e),
                               path      = self._CONFIG.get(i, p))

    @staticmethod
    def _set_dynamic_executable(executable, path):
        '''
        Manual configuration of the {Executable}.

        @param:    executable
        @pdef:     name of the executable file
        @ptype:    {String}

        @param:    path
        @pdef:     path to the executable file
        @ptype:    {String}
        '''
        ExternalExe._EXE = Executable(executable = executable, path = path)



'''
@file: CDhitList.py

@author: Jaume Bonet
@mail:   jaume.bonet@gmail.com
@date:   2013

@ [oliva's lab](http://sbi.imim.es)

@class: CDhitList
'''
import re

from SBI.beans      import StorableObject
from SBI.beans      import File
from .CDhit         import CDhit
from .CDhitHomolog  import CDhitHomolog


class CDhitList(StorableObject):
    '''
    List of cd-hit clusters.

    '''
    def __init__(self, cdhit_file = None):
        '''
        @param:    cdhit_file
        @pdef:     name of the cd-hit output file
        @pdefault: _None_. Create an empty list
        @ptype:    {String}

        '''
        self._clusters  = []
        self._allseqids = {}
        if cdhit_file is not None:
            self._file  = File(file_name = cdhit_file)
        else:
            self._file  = None

        if self._file is not None:
            self._parse_file()

    ##############
    # ATTRIBUTES #
    ##############
    @property
    def clusters(self):
        '''
        List of cd-hit clusters.

        @return: {List} of {CDhit}
        '''
        return self._clusters

    ###########
    # METHODS #
    ###########
    def get_cluster4sequence(self, sequence):
        '''
        Retrieve a cluster for a given sequence. _None_ if the sequence is not
        found.

        @param:    sequence
        @pdef:     name of the query sequence
        @ptype:    {String}

        @return: {CDhit}
        '''
        if sequence in self._allseqids:
            return self._clusters[self._allseqids[sequence]]
        else:
            return None

    def is_in_cluster(self, sequence):
        '''
        Evaluate if the sequence is in a cluster.

        @param:    sequence
        @pdef:     name of the query sequence
        @ptype:    {String}

        @return: {String} as 'N' if no, 'H' if yes and 'M' if cluster master
        '''
        c = self.get_cluster4sequence(sequence)
        if c is None:
            return 'N'
        else:
            return 'M' if c.is_master(sequence) else 'H'

    def add_cluster(self, cluster):
        '''
        Add a cd-hit cluster to the object.

        @param:    cluster
        @pdef:     new cd-hit cluster to add
        @ptype:    {CDhit}
        '''
        self._clusters.append(cluster)

    def add_sequence2cluster(self, sequence, cluster_id = None):
        '''
        Add a new sequence to a given cluster.

        @param:    sequence
        @pdef:     name of the query sequence
        @ptype:    {String}

        @param:    cluster_id
        @pdef:     identifier of the cluster
        @pdefault: _None_. Refers to the last added cluster.
        @ptype:    {String}
        '''
        if cluster_id is None:
            self.clusters[-1].add_sequence(sequence)
            self._allseqids[sequence.name] = len(self.clusters) - 1
        else:
            for x in range(len(self._clusters)):
                if self._clusters[x].identifier == cluster_id:
                    self._clusters[x].add_sequence(sequence)
                    self._allseqids[sequence.name] = x
                    break

    def dictionary_role_summary(self):
        '''
        Creates a dictionary separating master sequences and homolog sequences.

        @return: {Dictionary}
        '''
        data = {'master': [], 'homolog': []}
        for c in self.clusters:
            data['master'].append(c.master.name)
            for s in c.sequences:
                data['homolog'].append(s)
        return data

    def merge_clusters(self, cluster_file):
        '''
        When using an intermediate state to cluster by homology,
        the result of the second clustering is a clustering of clusters.
        We need to transform this into the original sequences

        @param:    cluster_file
        @pdef:     name of the second-step cluster output
        @ptype:    {String}
        '''
        clustlist  = CDhitList(cluster_file)
        newlist    = CDhitList()
        cluster_re = re.compile('Cluster\s+(\d+)')
        for cl in clustlist.clusters:
            c = CDhit(cluster_id = cl.identifier)
            newlist.add_cluster(c)
            cnum = int(cluster_re.search(cl.master.name).group(1))
            oldclust = self.clusters[cnum]
            newlist.add_sequence2cluster(sequence = oldclust.master)
            for s in oldclust.sequences:
                newlist.add_sequence2cluster(sequence = oldclust.sequences[s])
            for s in cl.sequences:
                idclust = cl.sequences[s]
                cnum = int(cluster_re.search(idclust.name).group(1))
                oldclust = self.clusters[cnum]
                master   = oldclust.master
                master.homology = idclust.homology
                newlist.add_sequence2cluster(sequence = master)
                for s in oldclust.sequences:
                    h = oldclust.sequences[s]
                    h.homology = int(h.homology * float(idclust.homology)/10)
                    newlist.add_sequence2cluster(sequence = h)

        self._clusters  = newlist._clusters
        self._allseqids = newlist._allseqids

    ###################
    # PRIVATE METHODS #
    ###################
    def _parse_file(self):
        '''
        Read the cd-hit output file into a {CDhitList}

        '''
        homolog_re = re.compile('(\d+)aa,\s+\>([\s\w]+)\.{3}')
        for line in self._file.read():
            if line.startswith('>'):
                c = CDhit(cluster_id = line.split()[-1].strip())
                self.add_cluster(c)
            else:
                data = homolog_re.search(line)
                d = line.split()
                h    = CDhitHomolog(name     = data.group(2),
                                    length   = data.group(1),
                                    homology = d[-1])
                self.add_sequence2cluster(sequence = h)
        self._file.close()

    def __len__(self):
        return len(self._clusters)

    def __repr__(self):
        text = []
        for c in self.clusters:
            text.append('{0}'.format(c))
        return '\n'.join(text)



"""SCOPlink

author: jbonet
date:   02/2014

@oliva's lab
"""

"""
Import Standard Libraries
"""
import os, re
import urllib

"""
Dependences in SBI library
"""
from SBI.databases import SCOPftp
from SBI.beans.Path     import Path
from SBI.beans.File     import File

class SCOPlink(object):
    def __init__(self, local = None):
        self._local     = os.path.abspath(local)
        self._desc      = None
        self._rel       = None
        if local is not None:
            self.local = local

    """ATTRIBUTES"""
    @property
    def local(self):         return self._local
    @local.setter
    def local(self, value):
        self._local    = os.path.abspath(value)
        self._desc     = os.path.join(self._local, os.path.split(SCOPftp['desc'])[-1])
        self._rel      = os.path.join(self._local, os.path.split(SCOPftp['rel'])[-1])

    @property
    def source(self):
        return SCOPftp['show']

    @property
    def descriptions(self):
        dscFile = File(self._desc, 'r')
        for dsc_line in dscFile.descriptor:
            if not dsc_line.startswith('#'):
                yield dsc_line

    @property
    def relations(self):
        relFile = File(self._rel, 'r')
        for rel_line in relFile.descriptor:
            if not rel_line.startswith('#'):
                yield rel_line

    """BOOLEANS"""
    @property
    def has_local(self):    return self._local is not None

    """METHODS"""
    def download(self):
        if not self.has_local:
            raise NameError('A local SCOP database directory must be defined.')

        Path.mkdir(self.local)
        urllib.urlretrieve(SCOPftp['desc'], self._desc)
        urllib.urlretrieve(SCOPftp['rel'],  self._rel)

        return True



def start_transaction(): return "SET autocommit = 0;\nSTART TRANSACTION;\n"
def end_transaction():   return "COMMIT;\n"

from Source   import Source
from PDBeChem import Element, PDBeChem
from GOterm   import GOterm
from TaxID    import TaxID
from Uniprot  import Uniprot
from Enzyme   import Enzyme
from PDB      import PDB
from Drug     import Drug
from SCOP     import SCOP
from TM       import TM
from CDhit    import CDhit
from Arch     import Arch
from Subclass import Subclass, Cclass, Loop


'''
@file: HmmExe.py

@author: Jaume Bonet
@mail:   jaume.bonet@gmail.com
@date:   2014

@ [oliva's lab](http://sbi.imim.es)

@class: HmmExe
@class: HmmParser
@class: HmmError
'''
import os
import re
import time
import subprocess
import ConfigParser
from abc import ABCMeta

from SBI.external import ExternalExe
from SBI.beans    import Executable
from SBI.beans    import File
from SBI.sequence import Fasta
from HmmResult    import HmmResult
from HmmHit       import HmmHit
from SBI          import SBIglobals as SBIg


class HmmExe(ExternalExe):
    '''
    Executes [HMMER](http://hmmer.janelia.org/).
    See the [user guide](ftp://selab.janelia.org/pub/software/hmmer3/3.1b1/Userguide.pdf)
    to learn which parameters to add.

    In case a multi fasta is given as database but it is not formatted,
    it will automatically call the formatting of the file.
    '''
    _DBFORMATER = None
    _FORMATSDB  = ['.h3f', '.h3i', '.h3m', '.h3p']
    _BLOCKED    = frozenset(['--domtblout'])

    def __init__(self, database, overwrite = None, clean = True):
        '''
        @param:    database
        @pdef:     database to blast upon.
        @ptype:    {String}

        @param:    overwrite
        @pdef:     For writing actions. Decides whether it can overwrite an
                   existing file.
        @pdefault: _SBIglobals.overwrite_
        @ptype:    {Boolean}

        @param:    clean
        @pdef:     remove the temporary files after the data is read.
        @pdefault: _True_
        @pclash:   if _SBIglobals.debug_ is _True_, clean is _False_
        @ptype:    {Boolean}

        @raises: {HmmError}
        '''
        self._error    = HmmError()

        #hmmer executable configuration
        if HmmExe._EXE is None:
            self._set_default_executable('hmmer')
            HmmExe._DBFORMATER = HmmExe._CONFIG.get('hmmer', 'dbformatexe')

        # Local overwrite takes precedence over Global overwrite
        self._overwrite   = SBIg.decide_overwrite(overwrite)

        self._database    = self._check_database(os.path.abspath(database))
        self._clean_files = clean

        #Optional execution parameters
        self._parameters  = {'attr': [], 'flag': []}

    ##############
    # ATTRIBUTES #
    ##############
    @property
    def database(self):
        '''
        Database to hmmer against.

        @returns: {String}
        '''
        return self._database

    @property
    def overwrite(self):
        '''
        Overwrite old files.

        @returns: {Boolean}
        '''
        return self._overwrite

    @property
    def clean_files(self):
        '''
        Remove temporary files after process is finished.

        @returns: {Boolean}
        '''
        return self._clean_files

    ###########
    # METHODS #
    ###########
    @staticmethod
    def dynamic(executable, path, dbformater):
        '''
        Manually set the values for hmmer path, executable and db format.

        @param:    executable
        @pdef:     name of the executable file
        @ptype:    {String}

        @param:    path
        @pdef:     path to the executable file
        @ptype:    {String}

        @param:    dbformater
        @pdef:     name of the database format executable
        @ptype:    {String}
        '''
        HmmExe._set_dynamic_executable(executable, path)
        HmmExe._DBFORMATER = dbformater

    def add_attribute(self, attribute_value, attribute_id):
        '''
        Adds specific attributes to the hmmer execution.

        @param:    attribute_value
        @pdef:     value of the attribute to add.
        @ptype:    {String}. Transformed otherwise.

        @param:    attribute_id
        @pdef:     label of the attribute to add.
        @pclash:   label 'domtblout' is blocked, as they need to be
                   specified when creating the instance.
        @ptype:    {String}
        '''
        if attribute_id in HmmExe._BLOCKED:
            raise self._error.blocked_parameter(HmmExe._BLOCKED)

        self._parameters['attr'].append([str(attribute_value), attribute_id])

    def add_parameter(self, attribute_id):
        '''
        Adds specific flag to the hmmer execution.

        @param:    attribute_id
        @pdef:     label of the attribute to add.
        @pclash:   label 'domtblout' is blocked, as they need to be
                   specified when creating the instance.
        @ptype:    {String}
        '''
        if attribute_id in HmmExe._BLOCKED:
            raise self._error.blocked_parameter(HmmExe._BLOCKED)

        self._parameters['flag'].append(attribute_id)

    def execute_query_seq(self, sequenceID = None, sequence          = None,
                          hmmer_input_file = None, hmmer_output_file = None,
                          work_directory   = os.getcwd()):
        '''
        Execute BLAST given a query sequence.

        @param:    sequenceID
        @pdef:     name of the query sequence.
        @pdefault: 'QuerySequence'
        @pclash:   If sequence is not provided, it assumes that the sequenceID
                   belongs to a protein in the database and, thus, it searches
                   for it. Either sequenceID or sequence needs to be provided.
        @ptype:    {String}

        @param:    sequence
        @pdef:     query sequence.
        @pdefault: _None_
        @pclash:   Either sequenceID or sequence needs to be provided.
        @ptype:    {String}

        @param:    hmmer_input_file
        @pdef:     name of the temporary fasta file to use as HMMER input.
        @pdefault: job.pid + clock + .tmp.fa
        @ptype:    {String}

        @param:    blast_output_file
        @pdef:     name of the temporary HMMER output file.
        @pdefault: job.pid + clock + .hmmer.out
        @ptype:    {String}

        @param:    work_directory
        @pdef:     Directory to which the temporary files will be created.
        @pdefault: Current working directory.
        @ptype:    {String}

        @raises: {AttributeError} if neither sequenceID nor sequence are
                  provided or if sequenceID is a list of sequence names.
        @raises: {HmmError} in HMMER execution or output parsing errors.

        @returns: {HmmResult}
        '''
        if sequenceID is None or sequence is None:
            msg = 'Both a sequence and sequenceID are needed to perform the blast.'
            raise AttributeError(msg)

        if isinstance(sequenceID, (list, set, tuple)):
            msg = 'Blasts can only be executed one at a time due to parse restrictions.'
            raise AttributeError(msg)

        # All the sequence is unknown, it will crash blast
        if len(re.sub(r'[Xx]', '', sequence)) == 0:
            SBIg.warn(self, 'Created an empty BlastResult.')
            return HmmResult(query_name     = sequenceID,
                             query_sequence = sequence,
                             database       = self.database)

        file_prefixes = ".".join([str(os.getpid()), str(int(time.clock()*100000))])
        file_prefixes = os.path.join(work_directory, file_prefixes)
        tmp_input     = file_prefixes + ".tmp.fa"
        tmp_output    = file_prefixes + ".blast.xml.out"

        tmp_input  = tmp_input  if hmmer_input_file  is None else hmmer_input_file
        tmp_output = tmp_output if hmmer_output_file is None else hmmer_output_file

        QueryFasta = Fasta.build(file_name = tmp_input, sequence_id = sequenceID,
                                 sequence  = sequence,  force       = True)

        self._execute(input_file = QueryFasta, output_file = tmp_output)

        hmmer_result = self._parse_hmmer(sequenceID, sequence, tmp_output)

        self._clean([tmp_input, tmp_output])

        return hmmer_result

    def execute_query(self, query_file, hmmer_output_file = None,
                      work_directory = os.getcwd()):
        '''
        Execute HMMER given a query sequence.

        @param:    query_file
        @pdef:     Fasta file with the query sequence.
        @pdefault: 'QuerySequence'
        @ptype:    {String} or {File} or {Fasta}

        @param:    hmmer_output_file
        @pdef:     name of the temporary HMMER output file.
        @pdefault: query_file.prefix + job.pid + .hmmer.out
        @ptype:    {String}

        @param:    work_directory
        @pdef:     Directory to which the temporary files will be created.
        @pdefault: Current working directory.
        @ptype:    {String}

        @raises: {AttributeError} if query_file is multi-fasta.
        @raises: {HmmError} in HMMER execution or output parsing errors.

        @returns: {HmmResult}
        '''
        if isinstance(query_file, basestring) or isinstance(query_file, File):
            newFasta = Fasta(fasta_file = query_file)
        elif isinstance(query_file, Fasta):
            newFasta = query_file

        if newFasta.is_multifasta:
            msg = 'Hmmer can only be executed one at a time due to parsing restrictions.'
            raise AttributeError(msg)

        #All the sequence is unknown, it will crash blast
        newFasta.load()
        query_sequence = newFasta.sequence
        if len(re.sub(r'[Xx]', '', query_sequence.sequence)) == 0:
            SBIg.warn(self, 'Created an empty BlastResult.')
            return HmmResult(query_name     = query_sequence.id,
                             query_sequence = query_sequence.sequence,
                             database       = self.database)

        file_prefixes = ".".join([newFasta.file.prefix, str(os.getpid())])
        file_prefixes = os.path.join(work_directory, file_prefixes)
        tmp_output    = file_prefixes + ".hmmer.out"

        tmp_output = tmp_output if hmmer_output_file is None else hmmer_output_file

        self._execute(input_file = newFasta, output_file = tmp_output)

        hmm_result = self._parse_hmmer(newFasta.sequence.id,
                                       newFasta.sequence.sequence, tmp_output)

        self._clean([tmp_output, ])

        return hmm_result

    def clean_optional_parameters(self):
        '''
        Deletes all optional parameters added to the {HmmExe}.
        '''
        self._parameters = {'attr': [], 'flag': []}

    ###################
    # PRIVATE METHODS #
    ###################
    def _execute(self, input_file, output_file):
        '''
        Executes BLAST.

        @param:    input_file
        @pdef:     file with the sequence to blast. (SINGLE FASTA)
        @ptype:    {Fasta}

        @param:    output_file
        @pdef:     name of the blast output file.
        @ptype:    {String}

        @raises: {HmmError}
        '''
        if not os.path.isfile(output_file) or self.overwrite:
            self._EXE.backup_command()

            #Adding fixed blast parameters
            self._EXE.add_attribute(output_file, '--domtblout')

            # Adding optional parameters
            for parameter in self._parameters['attr']:
                self._EXE.add_attribute(parameter[0], parameter[1])
            for flag in self._parameters['flag']:
                self._EXE.add_parameter(flag)

            # Adding input-output
            self._EXE.add_parameter(self.database)
            self._EXE.add_parameter(input_file.file.full)

            try:
                self._EXE.execute()
                self._EXE.restore_command()
            except SystemError as e:
                raise self._error.hmmer_execution_failed(e)

    def _check_database(self, database):
        '''
        Ensures that the given database to hmmer upon exists and that it is
        formated for hmmer.
        I also sets the index file for the database if it exists.

        @param:    database
        @pdef:     database to hmmer upon.
        @ptype:    {String}

        @returns: {String}, name of the database
        '''
        # Database file does not exist
        if not os.path.isfile(database):
            return self._error.database_does_not_exist(database)

        # Database is not formated (if dbformatexe is added in the
        # configuration path it will be auto-formated)
        formatdb_files = []

        for sufix in HmmExe._FORMATSDB:
            if not os.path.isfile(database + sufix):
                formatdb_files.append(database + sufix)

        if len(formatdb_files) > 0:
            try:
                self._format_database(database)
            except ConfigParser.NoOptionError as e:
                raise self._error.no_hmmer_format_exe(e)
            except SystemError as e:
                raise self._error.wrong_db_format(database, e)

        return database

    def _parse_hmmer(self, query_name, query_sequence, hmmer_output_file):
        '''
        Calls the parsing of the blast output.

        @param:    query_name
        @pdef:     name of the query protein
        @ptype:    {String}

        @param:    query_sequence
        @pdef:     sequence that has been searched against the database.
        @ptype:    {String}

        @param:    hmmer_output_file
        @pdef:     output file of blast.
        @ptype:    {String}
        '''
        return HmmParser.parse(query_name, query_sequence, self.database,
                               hmmer_output_file)

    def _format_database(self, database):
        '''
        Executes the hmmer script to format the database.

        @param:    database
        @pdef:     database to blast upon.
        @ptype:    {String}
        '''
        SBIg.warn(self, 'Formating {0} for hmmer.'.format(database))

        dbexe  = Executable(executable = HmmExe._DBFORMATER,
                            path       = self._EXE.path)

        if self.overwrite:
            dbexe.add_parameter('-f')

        dbexe.add_parameter(database)

        SBIg.alert('debug', self, 'Executing command {0}\n'.format(dbexe))

        dbexe.execute()

    def _clean(self, files):
        '''
        If clean_files is _True_ and not in debug mode, it removes the
        temporary files.

        @param:    files
        @pdef:     list of files to remove
        @ptype:    {List}
        '''
        if self.clean_files and not SBIg.debug:
            for temp_file in files:
                os.unlink(temp_file)


class HmmParser(object):
    '''
    Processes a cd-hit output into a {HmmResult} object.
    '''
    __metaclass__ = ABCMeta

    @staticmethod
    def parse(query_name, query_sequence, database, hmmer_output_file):
        '''
        Processes a cd-hit output into a {HmmResult} object.

        @param:    query_name
        @pdef:     name of the query protein.
        @ptype:    {String}

        @param:    query_sequence
        @pdef:     sequence of the query protein.
        @ptype:    {String}

        @param:    hmmer_output_file
        @pdef:     output file from BLAST.
        @ptype:    {String}

        @raises: {HmmError} if there are problems while parsing the XML file.
        @returns: {HmmResult}
        '''
        e = HmmError()
        if not os.path.isfile(hmmer_output_file):
            raise e.no_output_file()

        hmm = HmmResult(query_name, query_sequence, database)
        readfile = ["sort", "-gk7", "-gk13", hmmer_output_file]
        p = subprocess.Popen(readfile, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        out, err = p.communicate()
        if err.strip() == '':
            for line in out.split('\n'):
                if not line.startswith('#') and line.strip() != "":
                    line_data = line.strip().split()
                    new_domain = HmmHit(name      = line_data[3],  accession = line_data[4],
                                        length    = line_data[5],
                                        query_lim = [line_data[17], line_data[18]],
                                        hit_lim   = [line_data[15], line_data[16]],
                                        values    = [line_data[6],  line_data[12]],
                                        repeats   = line_data[10])
                    hmm.add_hit(new_domain)
        else:
            raise e.parse_error()

        return hmm


class HmmError(Exception):
    '''
    Manages different errors that can occur during the blast execution or
    parsing.

    '''
    _MSG     = ''
    _EXE     = False
    _NOBLAST = 'Hmmer has NOT been executed.'

    def __init__(self):
        pass

    def database_does_not_exist(self, database):
        HmmError._EXE = False
        HmmError._MSG = '{0} is not a file.'.format(database)
        return self

    def wrong_db_format(self, database, e):
        HmmError._EXE = False
        HmmError._MSG = '{0} could not be formated for hmmer.'.format(database)
        HmmError._MSG += '\n' + e.message
        return self

    def no_hmmer_format_exe(self, e):
        HmmError._EXE = False
        HmmError._MSG = 'Executable to format hmmer is not specified.'
        HmmError._MSG += '\n' + e.message
        return self

    def hmmer_execution_failed(self, e):
        HmmError._EXE = False
        HmmError._MSG = 'HMMER execution failed.'
        HmmError._MSG += '\n' + e.message
        return self

    def blocked_parameter(self, param):
        HmmError._EXE = False
        HmmError._MSG = '{0} are blocked.'.format(param)
        return self

    def no_output_file(self):
        HmmError._EXE = False
        HmmError._MSG = 'HMMER output not found.'
        return self

    def parse_error(self):
        HmmError._EXE = True
        HmmError._MSG = 'An error occurred while parsing the HMMER output.'
        return self

    def __str__(self):
        d  = HmmError._MSG + '\n'
        d += HmmError._NOBLAST if not HmmError._EXE else ''
        return d



from BioLib.Tools.BioExceptions import *

class Loop(object):
    '''
    This class represents a loop
    '''
    def __init__(self, loop_id, start, end):
        self.loop_id = loop_id
        self.seq_start = start
        self.struct_start = None
        self.seq_end = end
        self.struct_end = None
        self.residues = []

    # GETTERS #

    def get_id(self):
        return self.loop_id
    def get_start(self):
        if self.struct_start:
            return self.struct_start
        return self.seq_start
    def get_end(self):
        if self.struct_end:
            return self.struct_end
        return self.seq_end
    def get_residues(self):
        return self.residues

    # SETTERS #   

    def set_residues(self, residues):
        self.residues = residues
    def add_residue(self, residue):
        self.residues.append(residue)
    def redefine_position(self, start, end):
        self.struct_start = start
        self.struct_end = end

    # CLASS METHODS #

    def get_interacting_residues(self, structure, c_type = 'CB', max_distance = 12, uniq = True, warnings = False):
        '''
        Get the interacting residues with an structure (that can be a Structure 
        object or a Loop object since they share __iter__ method)
        '''
        max_distance = float(max_distance)
        interacting_residues_self = []
        interacting_residues_structure = []
        interacting_distances = []
        for residue1 in self:
            for residue2 in structure:
                try:
                    distance = residue1.get_residue_distance(residue2, c_type)
                except ResidueDistanceError as e:
                    if warnings: sys.stderr.write(str(e))
                    continue
                if distance <= max_distance:
                    interacting_residues_self.append(residue1)
                    interacting_residues_structure.append(residue2)
                    interacting_distances.append(distance)
        if uniq:
            return list(set(interacting_residues_self)), list(set(interacting_residues_structure))
        else:
            return interacting_residues_self, interacting_residues_structure, interacting_distances

    def get_interacting_coverage(self, structure, c_type = 'CB', max_distance = 12, warnings=False):
        '''
        Get the coverage of a loop interacting with an structure (that can be a Structure
        object or a Loop object since they share get_residues method)
        '''
        total_residues = float(len(self))
        interacting_residues = len(self.get_interacting_residues(structure = structure, c_type = c_type, max_distance = max_distance, uniq = True, warnings = warnings)[0])
        return interacting_residues/total_residues

    def __len__(self):
        '''
        Length of residues
        '''
        return len(self.residues)

    def __iter__(self):
        '''
        Iterates over each residue
        '''
        for residue in self.residues:
            yield residue

    def __str__(self):
        return "%s, %d-%d" % (self.loop_id, self.get_start(), self.get_end())



'''
@file: CDhit.py

@author: Jaume Bonet
@mail:   jaume.bonet@gmail.com
@date:   2013

@ [oliva's lab](http://sbi.imim.es)

@class: CDhit.py
'''


class CDhit(object):
    '''
    Represents a cd-hit cluster.

    '''
    def __init__(self, cluster_id):
        '''
        @param:    cluster_id
        @pdef:     identifier of the cd-hit cluster
        @ptype:    {String}
        '''
        self._id        = cluster_id
        self._master    = None
        self._sequences = {}

    ##############
    # ATTRIBUTES #
    ##############
    @property
    def identifier(self):
        '''
        cluster identifier.

        @return: {String}
        '''
        return self._id

    @property
    def master(self):
        '''
        Name of the master sequence of the cluster.

        @return: {String}
        '''
        return self._master

    @property
    def sequences(self):
        '''
        list of the proteins belonging to the cluster.

        @return: {List}
        '''
        return self._sequences

    ############
    # BOOLEANS #
    ############
    def is_master(self, seq):
        '''
        Checks if a given sequence is the master protein.

        @param:    seq
        @pdef:     query sequence id
        @ptype:    {String}

        @return: {Boolean}
        '''
        return self._master.name == seq

    def is_sequence(self, seq):
        '''
        Checks if a given sequence belongs to the cluster

        @param:    seq
        @pdef:     query sequence id
        @ptype:    {String}

        @return: {Boolean}
        '''
        return seq in self._sequences

    ###########
    # METHODS #
    ###########
    def add_sequence(self, cdhit_homolog):
        '''
        Adds a new cd-hit homolog to the cd-hit cluster.

        @param:    cdhit_homolog
        @pdef:     new homolog to the cluster
        @ptype:    {CDhitHomolog}
        '''
        if cdhit_homolog.is_master:
            self._master = cdhit_homolog
        else:
            self._sequences[cdhit_homolog.name] = cdhit_homolog

    def __repr__(self):
        text = []
        text.append('Cluster {0.identifier}:'.format(self))
        text.append('\tMaster Sequence: {0.master}'.format(self))
        for s in self.sequences:
            text.append('\t\t{0}'.format(self.sequences[s]))
        return '\n'.join(text)



import gzip, os, ftplib, sys
from BioLib.Structure.Structure import Structure
from BioLib.Structure.Residue import Residue
from BioLib.Structure.Atom import Atom

def read_pdb(pdb_file, one_model=True, from_string=False, merge_chains=False):
    '''
    Parses a PDB file
    @one_model: If the PDB contains more than one model, it returns only the first one
    @from_string: pdb_file is a PDB string instead of a file name. For example: read from a pipe
    @merge_chains: Merge the chains of the PDB into one structure, instead of returning a chain list
    '''
    models = []
    chains = []
    uniprot_ref = {}

    last_structure = '#####'
    base_residue = 0
    base_atom = 0

    if from_string:                pdb_fo = pdb_file.splitlines()
    elif pdb_file.endswith('.gz'): pdb_fo = gzip.open(pdb_file)
    else:                          pdb_fo = open(pdb_file)

    for line in pdb_fo:
        # Read Database references
        if line.startswith('DBREF'):
            if line[26:31].strip() == 'UNP':
                uniprot_ref[line[12]] = line[42:53].strip()
        # New model found
        if line.startswith('ENDMDL'):
            if one_model: 
                break
            else:
                models.append(chains)
                chains = []
                last_structure = '#####'
                base_residue = 0
                base_atom = 0
        # Read structure data
        if line.startswith('ATOM'):
            if len(line[26].strip()) > 0:
                continue
            chain = line[21:22]
            if last_structure != chain:
                if merge_chains and last_structure != '#####':
                    base_residue += structure.get_last_residue().get_num()
                    base_atom += structure.get_last_residue().get_atoms()[-1].get_num()
                    last_structure = chain
                else:
                    structure = Structure(os.path.basename(pdb_file).split('.')[0], chain)
                    chains.append(structure)
                    last_structure = chain  
            residueNum = int(line[22:26].strip())+base_residue
            ridueType = line[17:20].strip()
            atomNum = int(line[6:12].strip())#+base_atom
            atomType = line[13:17].strip()
            atomCoords = (float(line[27:38].strip()), float(line[38:46].strip()), float(line[46:54].strip()))
            if len(structure) == 0 or residueNum != structure.get_last_residue().get_num():
                residue = Residue(residueNum, ridueType)
                structure.add_residue(residue)
            atom = Atom(atomNum, atomType, atomCoords[0], atomCoords[1], atomCoords[2])
            residue.add_atom(atom)
            
    if not from_string:
        pdb_fo.close()

    if len(models) != 0:
        for model in models:
            for chain in model:
                if chain.get_chain() in uniprot_ref:
                    chain.set_uniprot_ref(uniprot_ref[chain.get_chain()])
        return models
    elif len(chains) > 1:
        for chain in chains:
            if chain.get_chain() in uniprot_ref:
                chain.set_uniprot_ref(uniprot_ref[chain.get_chain()])
        return chains
    else:
        if chains[0].get_chain() in uniprot_ref:
            chains[0].set_uniprot_ref(uniprot_ref[chains[0].get_chain()])
        return chains[0] 

def write_pdb(structures, pdb_name, multi_chain=False, multi_model=False):
    '''
    Creates a PDB file
    @structures = Structure objects to print
    @multi_chain = If true, the pdb has more than one chain                                                            
     - Structures must be a list of structure objects 
        --> (structure1, ..., structureN)
    @multi_model = If true, the pdb has more than one model
     - Structures must be a list of structure objects or chains (lists of structure objects) 
        --> (structure1, ..., structureN) or ((structure1, ..., structureN), ..., (structure1, ..., structureN))
    '''
    pdbFilefd = open(pdb_name, 'w')
    countModel = 0
    pdbFilefd.write('HEADER     PDB automatically created\n')
    if multi_model:
        for model in structures:
            pdbFilefd.write('MODEL%s'%countModel.rjust(9))
            if multi_chain:
                for chain in model:
                    pdbFilefd.write(chain.__str__())
            else:
                pdbFilefd.write(model.__str__())
            pdbFilefd.write('ENDMDL\n')
            countModel += 1
    elif multi_chain:
        for chain in structures:
            pdbFilefd.write(chain.__str__())
    else:
        pdbFilefd.write(structures.__str__())
    pdbFilefd.write('END')
    pdbFilefd.close()

def download_pdb(client_directory, pdb_list):
    '''
    Downloads PDBs from internet
    '''
    ftp_server = "ftp.ebi.ac.uk"
    server_directory = "/pub/databases/rcsb/pdb-remediated/data/structures/divided/pdb/"

    ftp = ftplib.FTP(ftp_server)
    ftp.login()
    for pdb in pdb_list:
        pdb = pdb.lower()
        ftp.cwd(server_directory+pdb[1:3])
        pdb_fo = open(os.path.join(client_directory, pdb+'.pdb.gz'), 'wb')
        try:
            ftp.retrbinary("RETR pdb"+pdb+".ent.gz", pdb_fo.write)
        except:
            raise RuntimeError("ERROR: Unable to download %s\n" % pdb)
        pdb_fo.close()
    ftp.quit()




